package JavaCode;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.PriorityQueue;
import java.util.StringTokenizer;
import java.util.Vector;
import JavaCode.OligoFile.OligoFileTop;

/*
SLIDER: Local Short Reads DNA Sequence Alignment
Beta Release; Ver 0.1, 22th Mar. 2008
By: Nawar Malhis
Canada's Michael Smith Genome Sciences Centre
*/

public class ReferenceSequences {
	public class Map {
		public int       size;
		public String    fileName;
		public String    outf;
		public long[][]  scoreDistrebution;
		public char[]    charSeq;
		public boolean[] snpf;
		public int[][]   cnt;
		// public int[]     cntLost;
		public float[][][] Pb;
		public float[][] snpSc;
 
		public byte[][]  sq0;
		public int[][]   histogram;
		public int       maxCoverage;
		public Runtime   rt;
		public Map(int sz, String name, String fout) throws Exception {
			// System.err.println("Start Map 0\t" + sz);
			rt = Runtime.getRuntime();
			System.gc();
			fileName = name;
			outf  = fout;
			size  = sz;
			charSeq   = new char [size];
			cnt       = new int[3][size];
			// cntLost   = new int[size];
			Pb        = new float[2][4][size];
			snpSc     = new float[2][size];
			sq0       = new byte[2][size];
			histogram = new int[Parameters.mapHistSize + 1][Parameters.mapHistSize + 1];
			if (Parameters.snpRef) {
				int ad;
				scoreDistrebution = new long[2][Parameters.rangeScoreDist];
				for (int i = 0; i < Parameters.rangeScoreDist; i++) {
					scoreDistrebution[0][i] = scoreDistrebution[1][i] = 0;
				}
				snpf = new boolean[size];
				for (int i = 0; i < snpf.length; i++) {
					snpf[i] = false;
				}
				try {
					reader = new BufferedReader(new FileReader(Parameters.pathData + outf + ".snp.ref"));
					reader.readLine();
					reader.readLine();
					reader.readLine();
					reader.readLine();
					reader.readLine();
					ad = getAddress();
					// System.err.println(ad);
					while(ad >= 0) {
						snpf[ad] = true;
						ad = getAddress();
						// System.err.println(ad);
					}
					reader.close();
				} catch (Exception e) {
					System.err.println(">>>>>>>\t" + Parameters.pathData + outf + ".snp.ref was not found!!!!!");
					Parameters.snpRef = false;
					snpf = new boolean[1];
					scoreDistrebution = new long[1][1];
				}
			} else {
				snpf = new boolean[1];
				scoreDistrebution = new long[1][1];
			}

			char c;
			int sIdx;
			FileReader ch2File  = new FileReader(name);
			sIdx = 0;
			c = (char) ch2File.read();
			while(c != 10) {
				c = (char) ch2File.read();
			}
			while(ch2File.ready()) {
				c = (char) ch2File.read();
				if (c != 10 && c != 13) {
					// System.err.print(c);
					charSeq[sIdx] = c;
					sIdx++;
				}
			}
			// System.err.println("*");
			ch2File.close();
			for(int i = 0; i < size; i++) {
				cnt[0][i] = cnt[1][i] = 0;
			}
			for (int i0 = 0; i0 < Parameters.mapHistSize + 1; i0++) {
				snpSc[0][i0] = snpSc[1][i0] = 0;
				for (int i1 = 0; i1 < Parameters.mapHistSize + 1; i1++) {
					histogram[i0][i1] = 0;
				}
			}
	    	String mapFl = Parameters.pathDB + Parameters.DATABASE + fout + ".u0";
	    	FileInputStream 	fi    = new FileInputStream(mapFl);
	    	BufferedInputStream bi    = new BufferedInputStream(fi, 127000);
	    	DataInputStream		strIn = new DataInputStream(bi);
			for (int i = 0; i < size - Parameters.oMxSize; i++) {
				sq0[Parameters.F][i] = strIn.readByte();
				sq0[Parameters.R][i] = strIn.readByte();
			}
			strIn.close();
			bi.close();
			fi.close();
		}
		int x0;
		int x1;
		public void ceil() throws Exception {
			System.err.println("CEIL: " + size);
			maxCoverage = 0;
			for(int i = 0; i < size; i++) {
				if (cnt[0][i] > maxCoverage)
					maxCoverage = cnt[0][i];
				if (cnt[1][i] > maxCoverage)
					maxCoverage = cnt[1][i];
				
				x0 = cnt[0][i];
				x1 = cnt[1][i];
				if ( x0 >= Parameters.mapHistSize) {
					x0 = Parameters.mapHistSize;
				}
				if ( x1 >= Parameters.mapHistSize) {
					x1 = Parameters.mapHistSize;
				}
				histogram[x0][x1]++;
				
				// if ((x0 + x1) > 0)
					// System.err.println("*:" + x0 + ", " + x1);
			}
			// System.runFinalization();
			System.err.println(Parameters.pathOut + outf + ".histogram");
			BufferedWriter writer = new BufferedWriter(new FileWriter(Parameters.pathOut + "ScoreResults/" + outf + ".histogram"));
			for (int i0 = 0; i0 <= Parameters.mapHistSize; i0++) {
				for (int i1 = 0; i1 <= Parameters.mapHistSize; i1++) {
					writer.write(histogram[i0][i1] + "\t");
				}
				writer.write("\n");
			}
			writer.close();
			snpSc = null;
			Pb    = null;
			cnt   = null;
			System.gc();
			System.err.println("End Map");
		}
	}
	/*
	public class OligoFileTop implements Comparable<OligoFileTop>{
		// public boolean ok;
		public Oligo ol;
		public int   id;
		
		public OligoFileTop() throws Exception {
			ol = new Oligo();
			id = -1;
			// ok = true;
		}
		
		public int compareTo(OligoFileTop oft) {
			return ol.compareTo(oft.ol);
		}
		
		public void loadOligo(Oligo o) {
			ol.d[0]   = o.d[0];
			ol.d[1]   = o.d[1];
			ol.ch     = o.ch;
			ol.start  = o.start;
			ol.id1    = o.id1;
			ol.size   = o.size;
			ol.flg    = o.flg;
			ol.weight = o.weight;
	    	ol.u0     = o.u0;	    	
		}
	}
//*/
	static final int    A             = 0;          // 
	static final int    C             = 1;          // 
	static final int    G             = 2;          // 
	static final int    T             = 3;          // 
	static final int    N             = 4;          // 
    static final int[]  comp          = {T, G, C, A, N}; // used to convert int bp to its complement int
    static final char[] nucl          = {'A', 'C', 'G', 'T', 'N'}; // used to convert int to char
    static final int    ALL           = 1;
    static final int    CLEAN         = 0;
    static final int    MMSZ          = 31;
    static int[]        BPint;
    
	static final int    UM            = 0;         // 
	static final int    MM            = 1;         // 
	static final int    NM            = 2;         //
	static final int    DATA          = 0;
	static final int    DATB          = 1;

    static final int    non           = 2; // max number of Ns in an oligo
    static final int    wordSize      = 31; // length of each word
    static       int    subResult;

	static       int    ux            = 0;
	
	static public boolean display     = false;
	
	private long[][] code;                     // for coding c_oligo into oligo
	private int      idx;                      // index in the chromosome
	private int		 oId;                      // oligo id for those with low weight
	private byte     chr;                      // chromosome index
	// private int      olid;
	private int[]    nList;                    // a list of the Ns in oligo
	private int      nCounter;                 // number of unknown nuc in a seq.
	private String   c_oligo;
	private char[]   bak_c_oligo;
	private Oligo[]  oligo;
	
	private OligoFile ofile;
	private Map       map;
	private OligoFile ofOne;
	private OligoFile ofTwo;
	private OligoFile ofNMatch;
	private OligoFile ofMatch;

	private Seq      seq;
	private long     numberOfOligos;
	private double   weightOfAT;
	private double   weightOfCG;
	private long     numberOfUQ;
	private long     numberOfUD;
	public  int[]    refSeqSize;

	private int      npbp;
	private int[]    mmHist;

	private FileOutputStream fwPrb; 
	private BufferedOutputStream bwPrb;
	private DataOutputStream filePrb;

	private FileInputStream fiPrb; 
	private BufferedInputStream biPrb;
	private DataInputStream strInPrb;

	private BufferedReader reader;

	private Vector<Oligo>[] bunk; 
	private byte[][] idArray; 
	private int   IdCount;
	
	static public int  getBpInt(char b) {
		// System.err.print(b);
    	return BPint[java.lang.Character.toUpperCase(b) - 'A'];
    }
    static public char getBpChar(int i) {
    	return nucl[i];
    }

    public  void finalize()  throws Exception {
	}
	public  ReferenceSequences() throws Exception {

		nList       = new int[Parameters.dbOligoSize];
		c_oligo     = new String();
		bak_c_oligo = new char[Parameters.dbOligoSize+1];
		
		oligo       = new Oligo[2];
		oligo[DATA] = new Oligo();
		oligo[DATB] = new Oligo();
		
		fillCode();
		
		bunk        = new Vector[2]; 
		bunk[0]     = new Vector<Oligo>(); 
		bunk[1]     = new Vector<Oligo>(); 

		numberOfOligos = 0;
		weightOfAT     = 0;
		weightOfCG     = 0;

		BPint = new int['Z' - 'A' + 1];
		for (char c = 'A'; c <= 'Z'; c++) {
			BPint[c-'A'] = -1;
		}
		BPint['A'-'A'] = A;
		BPint['C'-'A'] = C;
		BPint['G'-'A'] = G;
		BPint['T'-'A'] = T;
		seq = new Seq();
	}
	
	int getSkip(String fl) throws Exception {
		int s = 0;
		int count = 0;
		int[] x = {0, 0, 0, 0};
		int[] st= {0, 0, 0, 0};
		float stTotal = 0;
		BufferedReader reader = new BufferedReader(new FileReader(fl));
		for (String line = reader.readLine(); line != null; line = reader.readLine()) {
			if (count >= 1000)
				break;
			StringTokenizer tokens = new StringTokenizer(line);
		    for (int ii = 0; ii < 4; ii++) {
		    	x[ii] = Integer.parseInt(tokens.nextToken(), 10);
		    }
		    int b =  0;
		    for(int ii = 1; ii < 4; ii++) {
		    	if (x[ii] > x[b])
		    		b = ii;
		    }
		    st[b]++;
		}
		float min = 1;
		float max = 0;
		float tmp;
		stTotal = st[0] + st[1] + st[2] + st[3];
		for(int i = 0; i < 4; i++) {
			tmp = ((float) st[i])/stTotal;
			if (tmp > min)
				min = tmp;
			if(tmp < max)
				max = tmp;
			System.err.print(Parameters.trim(tmp) + "%\t");
		}
		if (min / max < 0.25)
			s = 1;
		reader.close();
		return s;
	}
	
	/*
	public  void generateOligosFromPRB_SET(String prbPath, String outFile) throws Exception {
		OligoFile outOligoFile;
		int  id   = Parameters.maxId;
		int  idQc = 0;
		boolean  used = false;
		int rd;
		long totalIn  = 0;
		long totalOut = 0;
		long totalMPS = 0;
		long totalCrisp  = 0;
		long totalLQ  = 0;
		boolean mps = false;
		boolean uq  = false;
		int  ok;
		int sz;
		
		numberOfOligos = 0;
		weightOfAT     = 0;
		weightOfCG     = 0;
		clearMeta();
		
		int x;
		outOligoFile = new OligoFile(outFile + ".ol" + ux, "", OligoFile.W);

		openPrbOut(outFile); 
		System.err.println(prbPath);
		File fl = new File(prbPath);
		File[] dirs = fl.listFiles();
		for (int dd = 0; dd < dirs.length; dd++) {
			if(dirs[dd].isDirectory()) {
		        File[] files = dirs[dd].listFiles();
		        for(int i=0; i<files.length; i++) {
					System.gc();
		            if(!files[i].isDirectory() && (files[i].toString()).indexOf("prb") >= 0) {
		            	if (Parameters.skipValue)
		            		Parameters.skipCycles = getSkip(" " + files[i]);
		            	System.err.print(i + "\t" + files[i] + "\tSkip: " + Parameters.skipCycles + "\t");
		    			BufferedReader reader = new BufferedReader(new FileReader(files[i]));
		    			for (String line = reader.readLine(); line != null; line = reader.readLine()) {
		    				StringTokenizer tokens = new StringTokenizer(line);
		    				sz = tokens.countTokens()/4 - Parameters.skipCycles;
		    				if (sz < Parameters.oMxSize)
		    					seq.size = sz;
		    				else
		    					seq.size = Parameters.oMxSize;
		    				// ------------------------------------------------ skip
		    				for (int sk = 0; sk < Parameters.skipCycles; sk++) {
		    			    	for (int ii = 0; ii < 4; ii++) {
		    			    		tokens.nextToken();
		    			    	}
		    				}
		    		    	// ------------------------------------------------
		    				for (int is = 0; is < seq.size; is++) {
		    					for (int js = 0; js < 4; js++) {
		    						x = Integer.parseInt(tokens.nextToken(), 10);
		    						seq.q[0][is][js] = (byte) x;
		    					}
		    				}
		    				ok = seq.solvePrbLine(0);
		    				totalIn++;
		    				if (ok < 3) {
		    					uq = false;
			    				if (ok == 0) {
			    					seq.getPS();  // equal sizes for now
	    							uq = true;
			    				} else if (ok == 1) {
			    					seq.getPS(); // must be short one
		    						Oligo.sortIndex = Oligo.WEIGHT;
		    						Collections.sort(seq.bunk);
		    						Oligo.sortIndex = Oligo.SEQ;
			    				} else if (ok == 2) {
			    					seq.getMPS(); // must be short one
			    				} 
		    					mps = true;
	    						Iterator<Oligo> it = seq.bunk.iterator();
	    						rd = 0;
	    						while (it.hasNext()) {
    								if (rd > Parameters.maxMu) {
    									break;
    								} 	    							
    								oligo[DATA] = (Oligo) it.next ();
	    							if (oligo[DATA].weight > Parameters.minWeight) {
		    							oligo[DATA].setMP(mps);
		    							oligo[DATA].setUQ(uq);
			    						if (mps) {
			    							totalMPS++;
			    							if(uq)
			    								totalCrisp++;
			    							mps = false;
			    						}
    									oligo[DATA].id1 = id;
    									oligo[DATA].setSide(0);
    	    							oligo[DATA].setLQ(false);
    									outOligoFile.sortWriteOligo(new Oligo(oligo[DATA]));
    									used = true;
    									totalOut++;
	    							} else {
	    								break;
	    							}
									rd++;
	    						}
	    						if (used) {
	    							seq.id = id;
	    							writePrb((byte) 0);	    					    		
	    							id--; // ++
	    						} else {
		    						idQc++;
	    						}
	    						used = false;
	    					} else {
    							seq.getMPS();
    							Iterator<Oligo> itmps = seq.bunk.iterator();
    							oligo[DATA] = (Oligo) itmps.next ();
    							if (oligo[DATA].weight > (Parameters.minWeight * 10)) 
    							{
	    							oligo[DATA].setMP(true);
	    							oligo[DATA].setUQ(false); // 
	    							oligo[DATA].setLQ(true);
									oligo[DATA].setSide(0);
									oligo[DATA].id1 = id;
    								outOligoFile.sortWriteOligo(new Oligo(oligo[DATA]));
	    							seq.id = id;
	    							writePrb((byte) 0);	    					    		
	    							id--; // ++
									totalOut++;
	    							totalMPS++;
	    							totalLQ++;
    							} else {
    								idQc++;
    							}
    							itmps = null;
	    					}
		    			}
		    			reader.close();
						System.err.println("\ttotalMPS: " + totalMPS + "\ttotalCrisp: " + totalCrisp + "\ttotalLQ: " + totalLQ + "\ttotal: " + totalOut);
		            }
				} 

			}
		} 
		System.err.println("minID:\t" + id);
		closePrbOut();
		Oligo.sortIndex = Oligo.SEQ;
		outOligoFile.saveSBunk(Oligo.SEQ);
		outOligoFile.minID = id;
		outOligoFile.dfn = outOligoFile.dfns;
		outOligoFile.close();
		outOligoFile = new OligoFile(outFile + ".ol" + ux, "", OligoFile.R);
		if (Parameters.contDB) {
			outOligoFile.joinFilterContaminationSort();
		} else {
			outOligoFile.joinSort();
		}
		outOligoFile.close();
	    System.out.println(
	    		"\nTotal lines .....................:\t" + totalIn + 
	    		"\nLines removed by QC .............:\t" + idQc + 
	    		"\nNumber of MPS only lines ........:\t" + totalMPS + 
	    		"\nNumber of UQ lines ..............:\t" + totalCrisp +
	    		"\nFinal number of Oligos ..........:\t" + totalOut +
	    		"\nMultiplication ratio of UD Oligos:\t" + ((float) (totalOut - (totalCrisp + totalLQ)) / (float) (Parameters.maxId - id - (totalCrisp + totalLQ)))
	    		);
	}
	//*/
	
	public void generateOligosFromPRB_SET2(String[] prbFiles, int nid, int clusterSize) throws Exception {
		if (clusterSize > 0 && clusterSize < nid) {
			System.err.println("node " + nid + " greater than cluster size " + clusterSize);
			return;
		}
		//  ring[] filesList;
		int numberOfPrbFiles;
		OligoFile outOligoFile;
		int  id   = Parameters.maxId;
		int  idQc = 0;
		boolean  used = false;
		int rd;
		long totalIn  = 0;
		long totalOut = 0;
		long totalMPS = 0;
		long totalCrisp  = 0;
		long totalLQ  = 0;
		boolean mps = false;
		boolean uq  = false;
		int  ok;
		int sz;
		int x;
		
		numberOfOligos = 0;
		weightOfAT     = 0;
		weightOfCG     = 0;
		clearMeta();
		String outOlFile;
		String outPrb;
		if (clusterSize == 0) {
			outOlFile = Parameters.pathData + Parameters.outFile + ".ol" + ux;
			outPrb    = Parameters.pathData + Parameters.outFile;
		} else {
			outOlFile = Parameters.pathParallel + Parameters.outFile + "." + nid + ".ol" + ux;
			outPrb    = outOlFile + "/" + Parameters.outFile;			
		}
		outOligoFile = new OligoFile(outOlFile, "", OligoFile.W);

		// System.err.println(outPrb);
		openPrbOut(outPrb);
		x = 0;
		//----------------------------------------------------------------
        for(int i=0; i<prbFiles.length; i++) {
        	if (clusterSize == 0 || (i) % clusterSize == (nid - 1)) {
            	if (Parameters.skipValue)
            		Parameters.skipCycles = getSkip(prbFiles[i]);
            	System.err.print(i + "\t" + prbFiles[i] + "\tSkip: " + Parameters.skipCycles + "\t");
    			BufferedReader reader = new BufferedReader(new FileReader(prbFiles[i]));
    			for (String line = reader.readLine(); line != null; line = reader.readLine()) {
    				StringTokenizer tokens = new StringTokenizer(line);
    				sz = tokens.countTokens()/4 - Parameters.skipCycles;
    				if (sz < Parameters.oMxSize)
    					seq.size = sz;
    				else
    					seq.size = Parameters.oMxSize;
    				// ------------------------------------------------ skip
    				for (int sk = 0; sk < Parameters.skipCycles; sk++) {
    			    	for (int ii = 0; ii < 4; ii++) {
    			    		tokens.nextToken();
    			    	}
    				}
    		    	// ------------------------------------------------
    				for (int is = 0; is < seq.size; is++) {
    					for (int js = 0; js < 4; js++) {
    						x = Integer.parseInt(tokens.nextToken(), 10);
    						seq.q[0][is][js] = (byte) x;
    					}
    				}
    				ok = seq.solvePrbLine(0);
    				totalIn++;
    				if (ok < 3) {
    					uq = false;
        				if (ok == 0) {
        					seq.getPS();  // equal sizes for now
    						uq = true;
        				} else if (ok == 1) {
        					seq.getPS(); // must be short one
    						Oligo.sortIndex = Oligo.WEIGHT;
    						Collections.sort(seq.bunk);
    						Oligo.sortIndex = Oligo.SEQ;
        				} else if (ok == 2) {
        					seq.getMPS(); // must be short one
        				} 
    					mps = true;
    					Iterator<Oligo> it = seq.bunk.iterator();
    					rd = 0;
    					while (it.hasNext()) {
    						if (rd > Parameters.maxMu) {
    							break;
    						} 	    							
    						oligo[DATA] = (Oligo) it.next ();
    						if (oligo[DATA].weight > Parameters.minWeight) {
    							oligo[DATA].setMP(mps);
    							oligo[DATA].setUQ(uq);
        						if (mps) {
        							totalMPS++;
        							if(uq)
        								totalCrisp++;
        							mps = false;
        						}
    							oligo[DATA].id1 = id;
    							oligo[DATA].setSide(0);
    							oligo[DATA].setLQ(false);
    							outOligoFile.sort_WriteOligo(new Oligo(oligo[DATA]), Oligo.SEQ);
    							used = true;
    							totalOut++;
    						} else {
    							break;
    						}
    						rd++;
    					}
    					if (used) {
    						seq.id = id;
    						writePrb((byte) 0);	    					    		
    						id--; // ++
    					} else {
    						idQc++;
    					}
    					used = false;
    				} //*
    				else {
    					seq.getMPS();
    					Iterator<Oligo> itmps = seq.bunk.iterator();
    					oligo[DATA] = (Oligo) itmps.next ();
    					if (oligo[DATA].weight > (Parameters.minWeight * 10)) 
    					{
    						oligo[DATA].setMP(true);
    						oligo[DATA].setUQ(false); // 
    						oligo[DATA].setLQ(true);
    						oligo[DATA].setSide(0);
    						oligo[DATA].id1 = id;
    						outOligoFile.sort_WriteOligo(new Oligo(oligo[DATA]), Oligo.SEQ);
    						seq.id = id;
    						writePrb((byte) 0);	    					    		
    						id--; // ++
    						totalOut++;
    						totalMPS++;
    						totalLQ++;
    					} else {
    						idQc++;
    					}
    					itmps = null;
    				} //*/
    			}
    			reader.close();
    			System.err.println("\ttotalMPS: " + totalMPS + "\ttotalCrisp: " + totalCrisp + "\ttotalLQ: " + totalLQ + "\ttotal: " + totalOut);
			// --------------------------------------------------------------------------
        	}
		} 
		// System.err.println("minID:\t" + id);
		closePrbOut();
		Oligo.sortIndex = Oligo.SEQ;
		outOligoFile.saveSBunk(Oligo.SEQ);
		outOligoFile.minID = id;
		outOligoFile.dfn = outOligoFile.dfns;
		outOligoFile.close();
		outOligoFile = new OligoFile(outOlFile, "", OligoFile.R);
		if (Parameters.contDB) {
			outOligoFile.joinFilterContaminationSort();
		} else {
			outOligoFile.joinSort();
		}
		outOligoFile.close();
	    System.out.println(
	    		"\nTotal lines .....................:\t" + totalIn + 
	    		"\nLines removed by QC .............:\t" + idQc + 
	    		"\nNumber of MPS only lines ........:\t" + totalMPS + 
	    		"\nNumber of UQ lines ..............:\t" + totalCrisp +
	    		"\nFinal number of Oligos ..........:\t" + totalOut +
	    		"\nMultiplication ratio of UD Oligos:\t" + ((float) (totalOut - (totalCrisp + totalLQ)) / (float) (Parameters.maxId - id - (totalCrisp + totalLQ)))
	    		);
	  //*/

	}

	public void generateOligosFromPRB_PET2(String[] prbFiles, int nid, int clusterSize) throws Exception {
		if (clusterSize > 0 && clusterSize < nid) {
			System.err.println("node " + nid + " greater than cluster size "
					+ clusterSize);
			return;
		}
		OligoFile outOligoFile;
		int id = Parameters.maxId;
		int oldId = id;
		int idQc = 0;
		boolean used = false;
		int rd;
		int ok = 0;
		long totalIn = 0;
		long totalOut = 0;
		long totalMPS = 0;
		long totalCrisp = 0;
		long totalLQ = 0;
		boolean mps = false;
		boolean uq;
		boolean idNotUsed;
		// int x;
		numberOfOligos = 0;
		weightOfAT = 0;
		weightOfCG = 0;
		clearMeta();
		//*
		String outOlFile;
		String outPrb;
		if (clusterSize == 0) {
			outOlFile = Parameters.pathData + Parameters.outFile + ".ol" + ux;
			outPrb    = Parameters.pathData + Parameters.outFile;
		} else {
			outOlFile = Parameters.pathParallel + Parameters.outFile + "." + nid + ".ol" + ux;
			outPrb    = outOlFile + "/" + Parameters.outFile;			
		}
		outOligoFile = new OligoFile(outOlFile, "", OligoFile.W);

		// System.err.println(outPrb);
		openPrbOut(outPrb); 
 
		// x = 0;

		for (int i = 0; i < prbFiles.length; i++) {
			if (clusterSize == 0 || (i) % clusterSize == (nid - 1)) {

				if (Parameters.skipValue)
					Parameters.skipCycles = getSkip(prbFiles[i]);
				System.gc();
				System.err.print(i + "\t" + prbFiles[i] + "\tSkip: "
						+ Parameters.skipCycles + "\t");
				BufferedReader reader = new BufferedReader(new FileReader(
						prbFiles[i]));
				for (String line = reader.readLine(); line != null; line = reader
						.readLine()) {
					idNotUsed = true;
					oldId = id;
					// seq.size = split(line);
					seq.size = split(line);
					for (byte l = 0; l < 2; l++) {
						// seq.nomm = 0;
						ok = seq.solvePrbLine(l);
						totalIn++;
						if (ok < 3) {
							uq = false;
							if (ok == 0) {
								seq.getPS();
								uq = true;
							} else if (ok == 1) {
								seq.getPS();
								Oligo.sortIndex = Oligo.WEIGHT;
								Collections.sort(seq.bunk);
								Oligo.sortIndex = Oligo.SEQ;
							} else if (ok == 2) {
								seq.getMPS(); // must be short one
							}
							mps = true;
							Iterator<Oligo> it = seq.bunk.iterator();
							rd = 0;
							used = false;
							while (it.hasNext() && rd <= Parameters.maxMu) {
								oligo[DATA] = (Oligo) it.next();
								if (oligo[DATA].weight > Parameters.minWeight) {
									oligo[DATA].setMP(mps);
									oligo[DATA].setUQ(uq);
									if (mps) {
										totalMPS++;
										if (uq) {
											totalCrisp++;
										}
										mps = false;
									}
									oligo[DATA].id1 = oldId;
									oligo[DATA].setSide(l);
									oligo[DATA].setLQ(false);
									// oligo[DATA].nsq = (byte) rd;
									outOligoFile.sort_WriteOligo(new Oligo(oligo[DATA]), Oligo.SEQ);
									used = true;
									totalOut++;
								} else {
									break;
								}
								rd++;
								seq.id = oldId;
								// writePrb(l); // original
								if (idNotUsed) {
									id--;
									idNotUsed = false;
								}
							}
							if (!used) {
								idQc++;
							} else {
								writePrb(l);
							}
						} else { // ===========================================================
							seq.getMPS();
							Iterator<Oligo> itmps = seq.bunk.iterator();
							oligo[DATA] = (Oligo) itmps.next();
							if (oligo[DATA].weight > (Parameters.minWeight * 10)) {
								oligo[DATA].setMP(true);
								oligo[DATA].setUQ(false); // 
								oligo[DATA].setLQ(true);
								oligo[DATA].setSide(l);
								oligo[DATA].id1 = id;
								outOligoFile.sort_WriteOligo(new Oligo(oligo[DATA]), Oligo.SEQ);
								seq.id = id;
								writePrb((byte) l);
								id--; // ++
								totalOut++;
								totalMPS++;
								totalLQ++;
							} else {
								idQc++;
							}
							itmps = null;
						} //*/
					}
				}
				reader.close();
				System.err.println("\tCrisp: " + totalCrisp + "\ttotalIDs: "
						+ (Parameters.maxId - id) + "\tMPS:\t" + totalMPS
						+ "\ttotalLQ:\t" + totalLQ + "\ttotal: " + totalOut);
			}
		}
		closePrbOut();
		Oligo.sortIndex = Oligo.SEQ;
		outOligoFile.saveSBunk(Oligo.SEQ);
		outOligoFile.minID = id - 1;
		outOligoFile.dfn = outOligoFile.dfns;
		outOligoFile.close();
		outOligoFile = new OligoFile(outOlFile, "", OligoFile.R);
		if (Parameters.contDB) {
			outOligoFile.joinFilterContaminationSort();
		} else {
			outOligoFile.joinSort();
		}
		outOligoFile.close();
		System.out.println("\nTotal lines .....................:\t"
				+ totalIn
				+ "\nLines removed by QC .............:\t"
				+ idQc
				+ "\nNumber of IDs ...................:\t"
				+ (Parameters.maxId - id)
				+ "\nNumber of MPS only sides ........:\t"
				+ (totalMPS)
				+ "\nNumber of Crisp sides ...........:\t"
				+ totalCrisp
				+ "\nFinal number of Oligos ..........:\t"
				+ totalOut
				+ "\nMultiplication ratio of UD Oligos:\t"
				+ ((float) (totalOut - totalCrisp) / (float) (Parameters.maxId
						- id - totalCrisp)));
		// */
	}
	
	/*
	public  void generateOligosFromPRB_PET(String prbPath, String outFile) throws Exception {
		OligoFile outOligoFile;
		int  id   = Parameters.maxId;
		int  oldId = id;
		int  idQc = 0;
		boolean used = false;
		int  rd;
		int  ok = 0;
		long totalIn  = 0;
		long totalOut = 0;
		long totalMPS = 0;
		long totalCrisp  = 0;
		long totalLQ  = 0;
		boolean mps = false;
		boolean uq;
		boolean idNotUsed;
		// boolean nmps = false;
		numberOfOligos = 0;
		weightOfAT     = 0;
		weightOfCG     = 0;
		clearMeta();
		
		outOligoFile = new OligoFile(outFile + ".ol" + ux, "", OligoFile.W);

		openPrbOut(outFile); 
		System.err.println(prbPath);
		File fl = new File(prbPath);
		File[] dirs = fl.listFiles();
		for (int dd = 0; dd < dirs.length; dd++) {
			if(dirs[dd].isDirectory()) {
		        File[] files = dirs[dd].listFiles();
		        for(int i=0; i<files.length; i++) {
		            if(!files[i].isDirectory()) {
		            	if (Parameters.skipValue)
		            		Parameters.skipCycles = getSkip("" + files[i]);
		            	System.gc();
		            	System.err.print(i + "\t" + files[i] + "\tSkip: " + Parameters.skipCycles + "\t");
		    			BufferedReader reader = new BufferedReader(new FileReader(files[i]));
		    			for (String line = reader.readLine(); line != null; line = reader.readLine()) {
		    				idNotUsed = true;
		    				oldId = id;
		    				// seq.size = split(line);
		    				seq.size = split(line);
		    				for (byte l = 0; l < 2; l++) {
			    				// seq.nomm = 0;
			    				ok = seq.solvePrbLine(l);  
			    				totalIn++;
			    				if (ok < 3) {
			    					uq = false;
			    					if(ok == 0) {
			    						seq.getPS();
			    						uq = true;
			    					} else if (ok == 1) {
			    						seq.getPS();
			    						Oligo.sortIndex = Oligo.WEIGHT;
			    						Collections.sort(seq.bunk);
			    						Oligo.sortIndex = Oligo.SEQ;
			    					} else if (ok == 2) {
				    					seq.getMPS(); // must be short one
				    				} 
			    					mps = true;
		    						Iterator<Oligo> it = seq.bunk.iterator();
		    						rd = 0;
		    						used = false;
		    						while (it.hasNext() && rd <= Parameters.maxMu) {
		    							oligo[DATA] = (Oligo) it.next();
		    							if (oligo[DATA].weight > Parameters.minWeight) {
			    							oligo[DATA].setMP(mps);
			    							oligo[DATA].setUQ(uq);
				    						if (mps) {
				    							totalMPS++;
				    							if(uq) {
				    								totalCrisp++;
				    							} 
			    								mps = false;
				    						}
	    									oligo[DATA].id1 = oldId;
	    									oligo[DATA].setSide(l);
	    									oligo[DATA].setLQ(false);
	    									// oligo[DATA].nsq = (byte) rd;
	    									outOligoFile.sortWriteOligo(new Oligo(oligo[DATA]));
	    									used = true;
	    									totalOut++;
		    							} else { 
		    								break;
		    							}
										rd++;		    						
		    							seq.id = oldId;
		    							writePrb(l);
    									if(idNotUsed) {
    		    							id--; 
    										idNotUsed = false;
    									}
		    						}
		    						if (!used) {
		    							idQc++;
		    						}
		    					} else { // ===========================================================
		   							seq.getMPS();
	    							Iterator<Oligo> itmps = seq.bunk.iterator();
	    							oligo[DATA] = (Oligo) itmps.next ();
	    							if (oligo[DATA].weight > (Parameters.minWeight * 10)) 
	    							{
		    							oligo[DATA].setMP(true);
		    							oligo[DATA].setUQ(false); // 
		    							oligo[DATA].setLQ(true);
										oligo[DATA].setSide(l);
										oligo[DATA].id1 = id;
	    								outOligoFile.sortWriteOligo(new Oligo(oligo[DATA]));
		    							seq.id = id;
		    							writePrb((byte) 0);	    					    		
		    							id--; // ++
										totalOut++;
		    							totalMPS++;
		    							totalLQ++;
	    							} else {
	    								idQc++;
	    							}
	    							itmps = null;
		    					}
		    				}
		    			}
		    			reader.close();
						System.err.println("\tCrisp: " + totalCrisp + "\ttotalIDs: " + (Parameters.maxId - id) + "\tMPS:\t" + totalMPS + 
								"\ttotalLQ:\t" + totalLQ + "\ttotal: " + totalOut);
		            }
				} // i files

			}
		} // dd
		closePrbOut();
		Oligo.sortIndex = Oligo.SEQ;
		outOligoFile.saveSBunk(Oligo.SEQ);
		outOligoFile.minID = id - 1;
		outOligoFile.dfn = outOligoFile.dfns;
		outOligoFile.close();
		outOligoFile = new OligoFile(outFile + ".ol" + ux, "", OligoFile.R);
		if (Parameters.contDB) {
			outOligoFile.joinFilterContaminationSort();
		} else {
			outOligoFile.joinSort();
		}
		outOligoFile.close();
	    System.out.println(
	    		"\nTotal lines .....................:\t" + totalIn + 
	    		"\nLines removed by QC .............:\t" + idQc + 
	    		"\nNumber of IDs ...................:\t" + (Parameters.maxId - id) + 
	    		"\nNumber of MPS only sides ........:\t" + (totalMPS) + 
	    		"\nNumber of Crisp sides ...........:\t" + totalCrisp +
	    		"\nFinal number of Oligos ..........:\t" + totalOut +
	    		"\nMultiplication ratio of UD Oligos:\t" + ((float) (totalOut - totalCrisp) / (float) (Parameters.maxId - id - totalCrisp)));
	  //
	}
//*/
	public  long statMPS(String prbPath) throws Exception {
		int[][] histogram = new int[2][Parameters.oMxSize + 1];
		int[][] mpsHis = new int[3][101]; // [0][] for all; [1][] for pass; [2][] for LQ
		float sprb;
		for(int i = 0; i < histogram[0].length; i++) {
			histogram[1][i] = histogram[0][i] = 0;
		}
		for(int i = 0; i < mpsHis[0].length; i++) {
			mpsHis[2][i] = mpsHis[1][i] = mpsHis[0][i] = 0;
		}
		long totalIn  = 0;
		long totalOut = 0;
		long sumUDin  = 0;
		long sumUDout = 0;
		long totalInOld   = 0;
		long totalOutOld  = 0;
		long sumUDinOld   = 0;
		long sumUDoutOld  = 0;
		long totalOutOld0 = 0;
		long totalInSub   = 0;
		long totalOutSub  = 0;
		long sumUDinSub   = 0;
		long sumUDoutSub  = 0;
		long GQ = 0;
		long LQ = 0;
		clearMeta();
		int[][]   histo  = new int[2][Parameters.oMxSize];
		float[][] cnt    = new float[2][Parameters.oMxSize];
		float[][] histoW = new float[2][Parameters.oMxSize];
		int[] sumLQBases = {0, 0};
		
		for (int i = 0; i < histo[0].length; i++) {
			histoW[1][i] = histoW[0][i] = cnt[1][i] = cnt[0][i] = histo[1][i] = histo[0][i] = 0;
		}
		BufferedWriter writerLst = new BufferedWriter(new FileWriter(prbPath + "stat.lst." + ((int) (100 * Parameters.seqSegNucP)) + "." + Parameters.maxUDN));
		BufferedWriter writerCluster = new BufferedWriter(new FileWriter(prbPath + "files.list"));
		writerLst.write("Dataset:\t" + Parameters.DATASET + "\n");
		writerLst.write("\tInLn\tinUD%\tRmvLn\trmvUD%\tOutLn\toutUD%\tOut0\n");

		int fid = 0;
		File fl = new File(prbPath);
		File[] dirs = fl.listFiles();
		for (int dd = 0; dd < dirs.length; dd++) {
			if(dirs[dd].isDirectory()) {
		        File[] files = dirs[dd].listFiles();
		        for(int i=0; i<files.length; i++) {
		            if(!files[i].isDirectory()) {
		            	System.err.print(i + "\t" + files[i] + "\t");
		            	writerCluster.write(files[i] + "\n");
		            	fid++;
		            	if (Parameters.skipValue)
		            		Parameters.skipCycles = getSkip("" + files[i]);
		    			BufferedReader reader = new BufferedReader(new FileReader(files[i]));
		    			for (String line = reader.readLine(); line != null; line = reader.readLine()) {
		    				if (Parameters.ETag == Parameters.PET) {
			    				seq.size = split(line);
			    				totalIn++;
			    				seq.solvePrbLine(0); 
			    				histogram[0][seq.totalUd]++;
		    					if(seq.totalUd <= Parameters.maxUDN) {
			    					sprb = seq.getP_MPS(histo, histoW, cnt, 0);
		    						GQ++;
			    					mpsHis[1][(int) (100 * sprb)]++;
		    					} else {
			    					sprb = seq.getP_MPS(histo, histoW, cnt, 1);
		    						if(sprb > (Parameters.minWeight * 10)) {
		    							histogram[1][seq.totalUd]++;
			    						LQ++;
				    					mpsHis[2][(int) (100 * sprb)]++;		    							
		    						}
		    					}
		    					mpsHis[0][(int) (100 * sprb)]++;
			    				totalIn++;
			    				seq.solvePrbLine(1); 
			    				histogram[0][seq.totalUd]++;
		    					// sprb = seq.getP_MPS(histo, histoW, cnt);
		    					if(seq.totalUd <= Parameters.maxUDN) {
			    					sprb = seq.getP_MPS(histo, histoW, cnt, 0);
		    						GQ++;
			    					mpsHis[1][(int) (100 * sprb)]++;
		    					} else {
			    					sprb = seq.getP_MPS(histo, histoW, cnt, 1);
		    						if(sprb > (Parameters.minWeight * 10)) {
		    							histogram[1][seq.totalUd]++;
			    						LQ++;
				    					mpsHis[2][(int) (100 * sprb)]++;		    							
		    						}
		    					}
		    					mpsHis[0][(int) (100 * sprb)]++;
		    				} else {
		    					seq.size = split(line);
			    				totalIn++;
			    				seq.solvePrbLine(0); 
			    				histogram[0][seq.totalUd]++;
		    					// sprb = seq.getP_MPS(histo, histoW, cnt);
		    					if(seq.totalUd <= Parameters.maxUDN) {
			    					sprb = seq.getP_MPS(histo, histoW, cnt, 0);
		    						GQ++;
			    					mpsHis[1][(int) (100 * sprb)]++;
		    					} else {
			    					sprb = seq.getP_MPS(histo, histoW, cnt, 1);
		    						if(sprb > (Parameters.minWeight * 10)) {
		    							histogram[1][seq.totalUd]++;
			    						LQ++;
				    					mpsHis[2][(int) (100 * sprb)]++;		    							
		    						}
		    					}
		    					mpsHis[0][(int) (100 * sprb)]++;
		    				}
		    			}
		    			reader.close();
		    			totalOut = 0;
		    			sumUDout = 0;
		    			sumUDin  = 0;
		    			for(int j = 0; j < histogram[0].length; j++) {
		    				sumUDin += j * histogram[0][j];
		    				if (j <= Parameters.maxUDN) {
		    					sumUDout += j * histogram[0][j];
		    					totalOut += histogram[0][j];
		    				}
		    			}
		    			totalInSub  = totalIn  - totalInOld;
		    			totalOutSub = totalOut - totalOutOld;
		    			sumUDinSub  = sumUDin  - sumUDinOld;
		    			sumUDoutSub = sumUDout - sumUDoutOld;
				        System.err.println();
						System.err.println("\tInLn\tinUD%\tRmvLn\trmvUD%\tOutLn\toutUD%\tOut0");
						System.err.print("\t" + totalInSub);
						System.err.print("\t" + Parameters.trim(((float) 100 * sumUDinSub)  / (totalInSub  * Parameters.oMxSize)) + "%");
						
						System.err.print("\t" + (totalInSub - totalOutSub));
						System.err.print("\t" + Parameters.trim(((float) 100 * (sumUDinSub - sumUDoutSub))  / ((totalInSub  - totalOutSub) * Parameters.oMxSize)) + "%");
						
						System.err.print("\t" + totalOutSub);
						System.err.print("\t" + Parameters.trim(((float) 100 * sumUDoutSub) / (totalOutSub * Parameters.oMxSize)) + "%");
						
						System.err.print("\t" + (histogram[0][0] - totalOutOld0));
						System.err.println("\t" + files[i]);
						
						writerLst.write("\t" + totalInSub);
						writerLst.write("\t" + Parameters.trim(((float) 100 * sumUDinSub)  / (totalInSub  * Parameters.oMxSize)) + "%");
						writerLst.write("\t" + (totalInSub - totalOutSub));
						writerLst.write("\t" + Parameters.trim(((float) 100 * (sumUDinSub - sumUDoutSub))  / ((totalInSub  - totalOutSub) * Parameters.oMxSize)) + "%");
						writerLst.write("\t" + totalOutSub);
						writerLst.write("\t" + Parameters.trim(((float) 100 * sumUDoutSub) / (totalOutSub * Parameters.oMxSize)) + "%");
						writerLst.write("\t" + (histogram[0][0] - totalOutOld0));
						writerLst.write("\t" + files[i] + "\n");
						totalOutOld0 = histogram[0][0];
						totalInOld  = totalIn;
						totalOutOld = totalOut;
						sumUDinOld  = sumUDin;
						sumUDoutOld = sumUDout;
			        }
		        } // i files
			} // if dd 
		} // dd dirs
		writerLst.close();
		writerCluster.close();
		BufferedWriter writerInfo = new BufferedWriter(new FileWriter(prbPath + "filesInfo.txt"));
		writerInfo.write("number of PRB files\t" + fid + "\n");
		writerInfo.close();
		BufferedWriter writer = new BufferedWriter(new FileWriter(prbPath + "stat." + ((int) (100 * Parameters.seqSegNucP)) + "." + Parameters.maxUDN));
		for (int i = 0; i < histo[0].length; i++) {
			sumLQBases[0] += histo[0][i];
			sumLQBases[1] += histo[1][i];
		}

		writer.write("Dataset:\t" + Parameters.DATASET + "\n");
		writer.write("total Sequences:\t" + totalIn + "\n");
		writer.write("Histogram number of UDP%:\n");
		for(int i = 0; i < histogram[0].length; i++) {
			writer.write(i + "\t" + ((float) histogram[0][i]) / totalIn + "\t" + histogram[0][i] + 
					"\t" + ((float) histogram[1][i]) / LQ + "\t" + histogram[1][i] + "\n");
		}
		
		writer.write("\nHistogram P_mps:\n");
		for(int i = 0; i < mpsHis[0].length; i++) {
			writer.write(i + "\t" + ((float) mpsHis[0][i]) / totalIn + "\t" + ((float) mpsHis[1][i]) / GQ + 
						"\t" + ((float) mpsHis[2][i]) / LQ + "\t" + mpsHis[0][i] + "\t" + mpsHis[1][i] + "\t" + mpsHis[2][i] + "\n");
		}
		writer.write("\n\tUD%\t\t\tWeighMPBt\t\tnumBasesPerUD"+ "\n");
		for (int i = 0; i < histo[0].length; i++) {
			writer.write(i + 
					"\t" + Parameters.trim(100 * ((float) (histo[0][i] + histo[1][i])/ (float) totalIn)) + 
					"\t" + Parameters.trim(100 * ((float) histo[0][i]/ (float) GQ)) + 
					"\t" + Parameters.trim(100 * ((float) histo[1][i]/ (float) (totalIn - GQ))) + 
					
					"\t" + Parameters.trim( 100 * (histoW[0][i] + histoW[1][i]) / (histo[0][i] + histo[1][i])) + 
					"\t" + Parameters.trim( 100 * histoW[0][i] / histo[0][i]) + 
					"\t" + Parameters.trim( 100 * histoW[1][i] / histo[1][i]) + 
					
					"\t" + Parameters.trim((cnt[0][i] + cnt[1][i]) / (histo[0][i] + histo[1][i])) + 
					"\t" + Parameters.trim(cnt[0][i] / histo[0][i]) + 
					"\t" + Parameters.trim(cnt[1][i] / histo[1][i]) + "\n");
		}
		writer.write("\n");
		writer.write("Input Data Set ....:\t" + Parameters.DATASET + "\n");
		writer.write("Lines In ..........:\t" + totalIn+ "\n");
		writer.write("%UDin .............:\t" + Parameters.trim(((float) 100 * sumUDin) / (totalIn  * Parameters.oMxSize)) + "%"+ "\n");
		writer.write("Lines Removed .....:\t" + (totalIn - totalOut) + "\tLQ\t" + LQ + "\n");
		writer.write("%UDin .............:\t" + Parameters.trim(((float) 100 * (sumUDin - sumUDout)) / ((totalIn  - totalOut) * Parameters.oMxSize)) + "%\n");
		writer.write("Candidate Lines Out:\t" + totalOut+ "\n");
		writer.write("sumLQBases ........:\t" + sumLQBases[0] + "\t" + sumLQBases[1] + "\n");
		writer.write("%UDout ............:\t" + Parameters.trim(((float) 100 * sumUDout) / (totalOut * Parameters.oMxSize)) + "%\n");
		writer.close();
		System.err.println();
		System.err.println("Input Data Set ....:\t" + Parameters.DATASET);
		System.err.println("Lines In ..........:\t" + totalIn);
		System.err.println("%UDin .............:\t" + Parameters.trim(((float) 100 * sumUDin) / (totalIn  * Parameters.oMxSize)) + "%");
		System.err.println("Lines Removed .....:\t" + (totalIn - totalOut) + "\tLQ\t" + LQ);
		System.err.println("%UDin .............:\t" + Parameters.trim(((float) 100 * (sumUDin - sumUDout)) / ((totalIn  - totalOut) * Parameters.oMxSize)) + "%");
		System.err.println("Candidate Lines Out:\t" + totalOut + "\t" + GQ);
		System.err.println("%UDout ............:\t" + Parameters.trim(((float) 100 * sumUDout) / (totalOut * Parameters.oMxSize)) + "%");
		return totalIn;
	}
	private  int  getMPBP(int i, int inc) {
		int mx = 0;
		npbp = mx + 1;
		for (int j = 1; j < 4; j++) {
			if (map.Pb[inc][mx][i] < map.Pb[inc][j][i]) {
				npbp = mx;
				mx = j;
			} else {
				if (map.Pb[inc][npbp][i] < map.Pb[inc][j][i])
					npbp = j;
			}
		}
		return mx;
	}
	/*
	public  float getBPP(int i, int bp, int inc) {
		if (bp < 0 || bp > 3) {
			System.err.println("bad bp:\t" + bp + "\ti:\t" + i);
			return 0;
		}
		if (i < 0) {
			System.err.println("bad i:\t" + i);
			return 0;
		}
		return map.Pb[inc][bp][i];
	}
	//*/
	private void openPrbOut(String fl) throws Exception {
		fwPrb     = new FileOutputStream(fl + ".prb");
		bwPrb     = new BufferedOutputStream(fwPrb);
		filePrb   = new DataOutputStream(bwPrb);
	}
	private void closePrbOut() throws Exception {
		filePrb.close();
		bwPrb.close();
		fwPrb.close();
	}
	private void writePrb(byte l) throws Exception {
		// System.err.println("Write PRB id:\t" + seq.id + "\t" + l);
    	filePrb.writeInt(seq.id);
    	filePrb.writeByte(l);
    	for (int ii = 0; ii < seq.size; ii++) {
    		for (int jj = 0; jj < 4; jj++) {
    			filePrb.writeByte(seq.q[l][ii][jj]);
    		}
    	}
	}
	private void openPrbIn(String f) throws Exception {
		fiPrb    = new FileInputStream(f); 
		biPrb    = new BufferedInputStream(fiPrb);
		strInPrb = new DataInputStream(biPrb) ;
    }
    private void closePrbIn() throws Exception {
    	strInPrb.close();
    	fiPrb.close();
    }
	private byte read_Prb(int id, int sid) throws Exception {
		byte l = -1;
		byte q;
		seq.size = Parameters.oMxSize;
		seq.id  = strInPrb.readInt();
		l = strInPrb.readByte();
		if (id < 0 ||(id == seq.id && sid == l)) { //
	    	for (int ii = 0; ii < Parameters.oMxSize; ii++) {
	    		for (int jj = 0; jj < 4; jj++) {
	    			q = strInPrb.readByte();
	    			seq.p[ii][jj] = (float) (1/(1+Math.pow(10,(((float)-q)/10))));
	    		}
	    	}
		} else {
	    	for (int ii = 0; ii < Parameters.oMxSize; ii++) {
	    		for (int jj = 0; jj < 4; jj++) {
	    			q = strInPrb.readByte();
	    		}
	    	}	
		}
		// System.err.println("-----> " + seq.id + "\t" + l);
    	return l;
	}
   
	public  void processReference(String[] files, int[] refSeqSize) throws Exception {
		//*
		ofile = new OligoFile(Parameters.pathDB + Parameters.DATABASE, "", OligoFile.W);
		for (chr = 0; chr < Parameters.SEQ_VAR.length; chr++) {//HUMAN_CHR.length
			processFile(Parameters.pathRefSeq + files[chr], refSeqSize, chr);
			// System.err.println(files[chr] + ":\t" + refSeqSize[chr]);
		}
		ofile.saveSBunk(Oligo.SEQ);
		ofile.dfn = ofile.dfns;
		ofile.close();
		//*/
		ofile = new OligoFile(Parameters.pathDB + Parameters.DATABASE, "", OligoFile.R);
		ofile.joinSortU2(); 
		ofile.close();
	}

	public  void processContamination(String pathConSeq, String ConDB) throws Exception {
		int oldDBSize = Parameters.dbOligoSize;
		/*
		if(Parameters.oMxSize <= Oligo.wordSize) {
			Parameters.dbOligoSize = Oligo.wordSize;
		}
		//*/
		Parameters.dbOligoSize = Parameters.oMxSize;
		int[] tmp = new int[1];
		try {
			File fl = new File(pathConSeq);
			System.err.println("ConDB:\t" + ConDB);
			ofile = new OligoFile(ConDB, "", OligoFile.W);
			System.err.println("pathConSeq:\t" + pathConSeq);
			File[] dirs = fl.listFiles();
			for (int dd = 0; dd < dirs.length; dd++) {
				if(dirs[dd].isDirectory()) {
			        File[] files = dirs[dd].listFiles();
			        for(int i=0; i<files.length; i++) {
						System.gc();
			            if(!files[i].isDirectory()) { // && (files[i].toString()).indexOf("prb") >= 0) {
			    			System.err.println("File:\t" + files[i].toString());
			    			processFile(files[i].toString(), tmp, 0);
			            }
			        }
				}
			}
			ofile.saveSBunk(Oligo.SEQ);
			ofile.dfn = ofile.dfns;
			ofile.close();
			Parameters.dbOligoSize = oldDBSize;
			ofile = new OligoFile(ConDB, "", OligoFile.R);
			ofile.joinSortU2(); 
			ofile.close();
			System.err.println("conamination sequences Database created");
		} catch(Exception e) {
			System.err.println("No conamination sequences");
			Parameters.contDB = false;
		}
	}
	
	private void processFile(String fn, int[] sz, int chr) throws Exception {
		char[] olc = new char[Parameters.dbOligoSize];
		oId = 0;
		// nof = 1;
		int c = 0;
		BufferedReader reader = new BufferedReader(new FileReader(fn));
		reader.readLine();
		clearMeta();

		for (int i = 0; i < Parameters.dbOligoSize; i++) {
			c = reader.read();
			while(c == 10 || c == 13) {
				c = reader.read();
			}
			
			olc[i] = Character.toUpperCase((char) c);
		}
		idx = 0;
		while (c > 0) {
			c_oligo = new String(olc);
			process(); // will create new DB line
			// olid--;
			c = reader.read();
			idx++;
			while(c == 10 || c == 13) {
				c = reader.read();
			}
			for (int i = 1; i < Parameters.dbOligoSize; i++) {
				olc[i-1] = olc[i];
			}
			olc[Parameters.dbOligoSize-1] = Character.toUpperCase((char) c);
			if (idx%1000000 == 0)
				System.err.println("\tidx\t" + idx/1000000 + "M");
		}
		reader.close();
		// closeStreamOut(DATA);
		c_oligo = new String(olc);
		sz[chr] = idx + Parameters.dbOligoSize;
	}
    private void process() throws Exception { 
    	char [] olc = c_oligo.toCharArray();
    	nCounter = 0;
    	for (int i = 0; i < Parameters.dbOligoSize; i++) {
    	    bak_c_oligo[i] = c_oligo.charAt(i);
    	    nList[i] = 0;
    	}

    	for (int i = 0; i < Parameters.dbOligoSize; i++) {
    	    if (olc[i] == 'N' || olc[i] == 'M' || olc[i] == 'R' || olc[i] == 'x') {
    		    nList[nCounter] = i;
    		    nCounter++;
    		    olc[i] = 'T';
    	    } 
    	}

    	if (nCounter == 0) {
    	    c_oligo = new String(olc);
    	    codeIt();
    	} else if (nCounter > non) {
    	    c_oligo = new String(bak_c_oligo);
    	    return;
    	} else {
    	    c_oligo = new String(olc);
		    codeIt();
    	    while (flipOligo() == 1) {
    		    codeIt();
    	    }
    	    oId++;
    	    for (int i = 0; i < Parameters.dbOligoSize; i++) {
    		    olc[i] = bak_c_oligo[i];
    	    }
    	}
	    c_oligo = new String(olc);
    }
    private void codeIt() throws Exception { // to fill an oligo object
    	boolean uq;
    	int ws;
    	Oligo ol;
    	// System.err.println("codeItOne");
    	// oligo[DATA].id1   = olid;
    	oligo[DATA].start = idx; // 
    	oligo[DATA].ch    = chr;
    	oligo[DATA].size = c_oligo.length();
    	for (int j = 0; j < Parameters.dbOligoSize; j += wordSize) {
    		if (j == 0) {
    			if (Parameters.dbOligoSize <= wordSize) {
    				ws = Parameters.dbOligoSize;
    			} else {
    				ws = wordSize;
    			}
    		} else {
    			ws = Parameters.dbOligoSize - wordSize;
    			/*
    			if ((Parameters.dbOligoSize - wordSize) <= wordSize) {
    				ws = Parameters.dbOligoSize - wordSize;
    			} else {
    				ws = wordSize;
    			}//*/    			
    		}
    		oligo[DATA].d[j/wordSize] = 0;
    		// System.err.println("j: " + j + "\tws: " + ws);
    		for (int i = 0; i < ws; i++) { 
    			oligo[DATA].d[j/wordSize] = oligo[DATA].d[j/wordSize] | 
    			code [getBpInt(c_oligo.charAt(j+i))]  [wordSize - i - 1];
    		}
    	}
    	oligo[DATA].weight = (float) 1.0/ (float) Math.pow(4.0,(double) nCounter);
    	oligo[DATA].id1 = oId;
    	if (oligo[DATA].weight < 0.97)
    		uq = false;
    	else
    		uq = true;
    	oligo[DATA].setUQ(uq);
    	ol     = new Oligo(oligo[DATA]); // ofile.writeRCOligo();
    	ol.id1 = 0; //olid;
    	ofile.sort_WriteOligo(new Oligo(ol), Oligo.SEQ);
    	// ol.reversComp();
    	ol.comp();
    	ol.revers();
    	// ol.start = ol.start + Parameters.dbOligoSize - 1;
    	ofile.sort_WriteOligo(ol, Oligo.SEQ);
    }

    private void addOligo(int fd) {
    	bunk[fd].add(new Oligo(oligo[fd]));
    }
   
    private int  flipOligo() { // to give different posible sequences of a seq., by changing Ns
    	char[] olc = c_oligo.toCharArray();
    	for (int i = nCounter - 1; i >= 0; i--) {
    	    if (getBpInt(olc[nList[i]]) == A) {
    	    	olc[nList[i]] = 'T';
    	    } else {
    	    	olc[nList[i]] = getBpChar(getBpInt(olc[nList[i]]) - 1);
    	    	c_oligo = new String(olc);
    		    // System.err.println("flipOligo:\t" + c_oligo + "\t" + nList[nCounter - 1]);
    		    return 1;
    	    }
    	}
    	c_oligo = new String(olc);
    	return -1;
    }
    
    private void deCodeIt(int od) { // to decode an oligo into C_oligo string
    	long a;
    	int sz = Oligo.now * wordSize + 1;
    	char[] olc = new char[sz + 1]; 
    	for (int w = 0; w < 2; w++) {
    		for (int i = 0; i < wordSize; i++) {
    			a = oligo[od].d[w];
    			a = a << (2 + 2 * i);
    			a = a >>> (62);
    			olc[w * wordSize + i] = getBpChar((int) a);
    		}
    	}
    	c_oligo = (new String(olc)).substring(0, oligo[DATA].size);
    }
    
    public  void align(String fDatabase, String pData, String pOut, String fData, int uCompare) throws Exception {
    	//             String db, String pd, String po, String dt, int osz, int ux
		seq = new Seq();
    	String fOne   = pData + fData + ".ol" + ux;
    	String fTwo   = fDatabase;
    	String mMatch = pOut + fData + ".m"  + ux;
    	String nMatch = pOut + fData + ".nm" + ux;
    	subResult = UM;
		//System.gc();
    	subtractSQ2f1(fOne, fTwo, nMatch, mMatch, uCompare);
    }
    
    public  void subtractSQ2f1(String fOne, String fTwo, String nMatch, String mMatch, int uCompare) throws Exception {
    	Oligo  bakO;
    	int mmIndex;
    	boolean u0bool;
    	int idxDbDat;
    	int compResult;
    	int dbsz;
    	int sz = 0;
    	int evSize;
    	int readCount = 0;
    	int S0Count   = 0;
    	int S1Count   = 0;
    	int petCount  = 0;
    	BufferedWriter multiMatchHist = null;
    	
    	mmHist   = new int[Parameters.mmHistSize];
    	ofOne    = new OligoFile(fOne, "", OligoFile.R);
    	evSize   = (int) (Parameters.maxId - ofOne.minID + 1); // + ofOne.weightOfCG);
    	ofTwo    = new OligoFile(fTwo, "", OligoFile.R);
    	if (subResult == NM) {
    		ofNMatch = new OligoFile(nMatch, "", OligoFile.W);
    	} else {
    		ofMatch  = new OligoFile(mMatch, "", OligoFile.W);
    	}
    	
    	for (int i = 0; i < mmHist.length; i++) {
    		mmHist[i] = 0;
    	}
    	
    	if (evSize > 0) {
        	idArray   = new byte[4][evSize];
    		for(int i = 0; i < evSize; i++) {
    			idArray[0][i] = idArray[1][i] = idArray[2][i] = idArray[3][i] = 0;
    		}
    	}
		idxDbDat = 0;
		idx      = 0;
		clearMeta();

		ofOne.readOligo(oligo[DATA]);
		sz = oligo[DATA].size;
		readBunk(DATA, sz); 
		ofTwo.readOligo(oligo[DATB]);
		readBunk(DATB, sz); 
		
		if (uCompare < 0) {
			uCompare = 0;
		}
    	if (uCompare == 0) {
    		u0bool = true;
    		multiMatchHist = new BufferedWriter(new FileWriter(Parameters.pathOut + "multiMatch.hist"));
    	} else
    		u0bool = false;
		System.err.print("\nSearching:\t");
		while(ofOne.available() > 0 && ofTwo.available() > 0) {
	    	bakO = new Oligo(oligo[DATA]); 
			compResult = (bunk[DATA].get(bunk[DATA].size()-1)).equal(bunk[DATB].get(bunk[DATB].size()-1), sz);

			if (compResult == 0) {
				dbsz = bunk[DATB].size();
				if (dbsz == 1) {
					if (subResult == UM) {
						Iterator<Oligo> it = bunk[DATA].iterator();
						Oligo oligoDB = new Oligo(bunk[DATB].get(0)); // ------------------------------------------ delete new?
						while (it.hasNext()) {
							oligo[DATA]        = (Oligo) it.next();
							oligo[DATA].setRC(oligoDB.isRC());
							oligo[DATA].weight = oligo[DATA].weight * oligoDB.weight;
							oligo[DATA].ch     = oligoDB.ch;
							oligo[DATA].start  = oligoDB.start;									
							if (oligoDB.isRC()) {
								oligo[DATA].start += oligoDB.size - oligo[DATA].size;
							} 
							idx++;
							oligo[DATA].setUM(true);
							oligo[DATA].setU0(u0bool);
							if (evSize > 0) {
								idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] = (byte) (idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] | (byte) 1);
								idArray[oligo[DATA].getSide()+2][Parameters.maxId - oligo[DATA].id1] += 1;
							}
							if (oligoDB.isRC()) {
								Oligo rco = (new Oligo(oligo[DATA])); //
								rco.reversComp();
								ofMatch.writeOligo(rco);
							} else {
								ofMatch.writeOligo(oligo[DATA]);
							}
						}
					}
				} else {
					if (subResult == UM) {
						if (u0bool) {
							if (dbsz < mmHist.length) {
								mmIndex = dbsz;								
							} else {
								mmIndex = mmHist.length - 1;
							}
						    if (u0bool) {
						    	mmHist[mmIndex] += (dbsz * bunk[DATA].size());
						    }
						} 
						if (dbsz <= Parameters.MMBRK) { 
							// ii = 0;
							// System.err.println("\ndbsz:\t" + dbsz);
						    Iterator<Oligo> idb = bunk[DATB].iterator();
					        for (; idb.hasNext(); ) {
					        	Oligo oligoDB = new Oligo((Oligo) idb.next()); // ----------------------------- delete new? 2027CAAXX
							    Iterator<Oligo> it = bunk[DATA].iterator();
						        for (; it.hasNext(); ) {
						    		oligo[DATA]        = (Oligo) it.next();
									oligo[DATA].setRC(oligoDB.isRC());
									oligo[DATA].weight = oligo[DATA].weight * oligoDB.weight / (float) dbsz;
									oligo[DATA].start  = oligoDB.start;									
									if (oligoDB.isRC()) {
										oligo[DATA].start += oligoDB.size - oligo[DATA].size;
									} 
									oligo[DATA].ch = oligoDB.ch;
									idx++;
									oligo[DATA].setUM(false);
									oligo[DATA].setU0(u0bool);
									if (evSize > 0)
										idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] = (byte) (idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1]|1);
									if (oligoDB.isRC()) {
										Oligo rco = (new Oligo(oligo[DATA])); //
										ofMatch.writeOligo(rco);
									} else {
										ofMatch.writeOligo(oligo[DATA]);
									}
								}
								// ii++;
						        /*
								if (ii > Parameters.MMBRK) { // ))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
									System.err.println("ii:\t" + ii);
									break;
								} //*/
						    }
						} else {
						    Iterator<Oligo> it = bunk[DATA].iterator();
						    // System.err.print("\ndbsz:\t" + dbsz + "\tdsz:\t" + bunk[DATA].size() + "\t");
					        while (it.hasNext()) {
					        	// System.err.print(".");
					    		oligo[DATA]        = (Oligo) it.next();
								if (evSize > 0)
									idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] = 
										(byte) (idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1]|1);
					        }
					        // System.err.println();
						}
			        }
				}
				oligo[DATA] = bakO; //new Oligo(bakO);
				readBunk(DATA, sz); 
				readBunk(DATB, sz); 
			} else if (compResult > 0) {
			    if (subResult == NM) {
			    	Iterator<Oligo> it = bunk[DATA].iterator();
			    	for (; it.hasNext(); ) {
			    		oligo[DATA] = new Oligo((Oligo) it.next());
				    	ofNMatch.writeOligo(oligo[DATA]);
				    }
		        }
				oligo[DATA] = bakO; //new Oligo(bakO);
				readBunk(DATA, sz); 
			} else {
				readBunk(DATB, sz); 
			}

			idxDbDat++;
			if (idxDbDat % 1000000 == 0) {// 1M 
				System.err.print(oligo[DATA].getBPat(0) + (oligo[DATA].getBPat(1) + " "));
				System.gc();
			}
		}
		System.err.println();
    	bunk[DATA].clear();
    	bunk[DATA].trimToSize();
    	bunk[DATB].clear();
    	bunk[DATB].trimToSize();
		ofOne.close();
		ofTwo.close();
		if (subResult == UM) {
			ofMatch.close();
		}
	    if (subResult == NM) {
	    	ofNMatch.close();
	    }
		ofNMatch = null;
		ofMatch  = null;
		ofOne    = null;
		ofTwo    = null;
    	System.gc();
    	int rdCount = 0;
    	int rdVal   = 0;
		if (evSize > 0) {
			IdCount = 0;
			for(int i = 0; i < evSize; i++) {
				if (idArray[2][i] > 1){
					rdCount++;
					rdVal += idArray[2][i] - 1;
				} 
				if (idArray[3][i] > 1){
					rdCount++;
					rdVal += idArray[3][i] - 1;
				} 
				if ((idArray[0][i]%2) == 1 || (idArray[1][i]%2) == 1){
					IdCount++;
				} 
				if ((idArray[0][i]%2) == 1){
					readCount++;
					S0Count++;
					if ((idArray[1][i]%2) == 1) {
						petCount++;
					}
				}
				if ((idArray[1][i]%2) == 1) {
					readCount++;
					S1Count++;
				}
			}
			if (u0bool) {
				multiMatchHist.write("i\tHist\n");
				for(int i = 0; i < mmHist.length; i++) {
					multiMatchHist.write(i + "\t" + mmHist[i] + "\n");
				}
			}
			if (subResult == UM) {
				System.err.println("S0       Count:\t" + S0Count);
				System.err.println("S1       Count:\t" + S1Count);
				System.err.println("S0 +  S1 Count:\t" + readCount);
				System.err.println("S0 || S1 Count:\t" + IdCount);
				System.err.println("S0 && S1 Count:\t" + petCount);
				System.err.println("redundent Count:\t" + rdCount);
				System.err.println("redundent Val :\t" + rdVal);
			}
		}
		if (u0bool) {
			multiMatchHist.close();
		}
    }
    
    public  void subtractID(String inf, String outf) throws Exception {
    	Oligo ol;
    	
		Oligo.sortIndex = Oligo.SEQ;
    	ofOne    = new OligoFile(inf, "", OligoFile.R);
    	ofile    = new OligoFile(outf , "", OligoFile.W);
		ofOne.readOligo(oligo[DATB]);
		while(ofOne.available() > 0) {
			if (oligo[DATB].weight > Parameters.cutWeight && !oligo[DATB].isLQ()) {
				if (idArray[oligo[DATB].getSide()][Parameters.maxId - oligo[DATB].id1] == 0) {
					deCodeIt(DATB);
					for (byte i = 0; i < oligo[DATB].size; i++) {
						for (int j = 0; j < 4; j++) {
							if(nucl[j] != c_oligo.charAt(i)) {
								ol = new Oligo(c_oligo.substring(0, i) + nucl[j] + c_oligo.substring(i+1, oligo[DATB].size));
								ol.id1    = oligo[DATB].id1;
								ol.flg    = oligo[DATB].flg;
								ol.weight = oligo[DATB].weight;
								ol.u0     = i;
								ofile.sort_WriteOligo(ol, Oligo.SEQ); 
								idx++;
							}
						}
					}
				}
			} 
			ofOne.readOligo(oligo[DATB]);
		}
		idArray = null;
		ofOne.close();
		ofOne = null;
		ofile.saveSBunk(Oligo.SEQ);
		ofile.dfn = ofile.dfns;
		ofile.close();
		ofile = null;
		System.gc();
		ofile = new OligoFile(outf, "", OligoFile.R);
		ofile.joinSort(); 
		ofile.close();
		ofile = null;
		System.gc();
    }
    
    private void readBunk(int fd, int sz) throws Exception {
    	byte two = 2;
    	OligoFile of;
    	if (fd == DATA)
    		of = ofOne;
    	else 
    		of = ofTwo;
    	
    	Oligo bak = new Oligo(oligo[fd]);
    	bunk[fd].clear();
    	if (fd == DATA && oligo[fd].isLQ()) {
    		idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] = (byte) (idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] | two);
    	}
    	addOligo(fd);
		while(of.readOligo(oligo[fd])) {
			if (bak.equal(oligo[fd], sz) != 0) {
				break;
			}
	    	if (fd == DATA && oligo[fd].isLQ()) {
	    		idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] = (byte) (idArray[oligo[DATA].getSide()][Parameters.maxId - oligo[DATA].id1] | two);
	    	}
	    	addOligo(fd);
		}   
    }

    /*
    public void combineFiles(String outPath, String outFile) throws Exception {
    	
		System.err.println(outPath);
		File fl = new File(outPath);
		File[] dirs = fl.listFiles();
		for (int dd = 0; dd < dirs.length; dd++) {
			if (dirs[dd].isDirectory()) {
				File[] files = dirs[dd].listFiles();
				int length = files.length;

				FileOutputStream filein2 = new FileOutputStream(outFile
						+ "-final.prb");
				DataOutputStream dataout2 = new DataOutputStream(filein2);

				int size2 = 0;
				for (int i = 0; i < length; i++) {
					if (files[i].toString().indexOf("prb") >= 0) {
						size2++;
					}
				}
				int[][] shift_array = new int[size2 + 1][3];

				for (int i = 1; i < size2 + 1; i++) {
					openPrbIn(outFile + "." + i + ".prb");
					System.out.println(outFile + "." + i + ".prb");
					int k = 0;
					int ID = 0;

					boolean EOF = false;
					while (!EOF) {
						try {
							ID = strInPrb.readInt();
							if (k == 0) {
								if (i == 1) {
									shift_array[i][0] = ID;
									shift_array[i][2] = 0;
								} else {
									shift_array[i][0] = shift_array[i - 1][1] - 1;
									shift_array[i][2] = ID
											- shift_array[i - 1][1] + 1;
								}
								dataout2.writeInt(shift_array[i][0]);
							} else {
								if (i == 1) {
									dataout2.writeInt(ID);
								} else {
									dataout2.writeInt(ID - shift_array[i][2]);
								}
							}

							Byte B1 = strInPrb.readByte();
							dataout2.writeByte(B1);
							for (int ii = 0; ii < seq.size; ii++) {
								for (int jj = 0; jj < 4; jj++) {
									Byte B2 = strInPrb.readByte();
									dataout2.writeByte(B2);
								}
							}
							k++;
						} catch (EOFException e) {
							EOF = true;
						}
					}
					if (i == 1) {
						shift_array[i][1] = ID;
					} else {
						shift_array[i][1] = ID - shift_array[i][2];
					}
					closePrbIn();
				}
				dataout2.close();

				int size = 0;
				for (int i = 0; i < length; i++) {
					if (files[i].toString().indexOf("ol") >= 0) {
						size++;
					}
				}
				File[] oligofiles = new File[size];

				int j = 0;
				for (int i = 0; i < length; i++) {
					if (files[i].toString().indexOf("ol") >= 0) {
						oligofiles[j] = files[i];
						j++;
					}
				}
				FileInputStream[] sfi = new FileInputStream[oligofiles.length];
				BufferedInputStream[] sbi = new BufferedInputStream[oligofiles.length];
				DataInputStream[] sstrIn = new DataInputStream[oligofiles.length];

				OligoFile CompleteOligoFile = new OligoFile(outFile
						+ ".ol-final", "", OligoFile.W);

				PriorityQueue<OligoFileTop> pQueue = new PriorityQueue<OligoFileTop>();
				OligoFileTop oft;
				int next;

				for (int i = 0; i < oligofiles.length; i++) {
					oft = new OligoFileTop();
					sfi[i] = new FileInputStream(dirs[dd] + "/"
							+ oligofiles[i].getName() + "/file1");
					sbi[i] = new BufferedInputStream(sfi[i], 5000);
					sstrIn[i] = new DataInputStream(sbi[i]);
					getOligo(oft.ol, i);
					oft.id = i;
					pQueue.add(oft);
				}

				while (pQueue.size() > 0) {
					oft = pQueue.poll();
					next = oft.id;
					if (oft.id != 0) {
						oft.ol.id1 = oft.ol.id1 - shift_array[oft.id + 1][2];
					}

					CompleteOligoFile.writeOligo(oft.ol);
					if (getOligo(oft.ol, next)) {
						oft.id = next;
						pQueue.add(oft);
					}
				}

				CompleteOligoFile.close();
				for (int i = 0; i < oligofiles.length; i++) {
					sstrIn[i].close();
					sbi[i].close();
					sfi[i].close();
				}
				CompleteOligoFile.writeMeta();
			}
		}
	}
    
    private boolean getOligo(Oligo ol, int frm) throws Exception {
        boolean ret = true;
        if (sstrIn[frm].available() > 0) {
                ol.d[0] = sstrIn[frm].readLong();
                ol.d[1] = sstrIn[frm].readLong();
                ol.ch = sstrIn[frm].readByte();
                ol.start = sstrIn[frm].readInt();
                ol.id1 = sstrIn[frm].readInt();
                ol.size = sstrIn[frm].readByte();
                ol.flg = sstrIn[frm].readByte();
                ol.u0 = sstrIn[frm].readByte();
                ol.weight = sstrIn[frm].readByte();
                ol.weight = ol.weight / 100;
        } else {
                ret = false;
        }
        return ret;
    }
    //*/
    
    public  void seperate(String[] files) throws Exception { // , int[][] resultCount
    	if (Parameters.ETag == Parameters.SET) {
       		seperate_SET(); // , resultCount
    	} else {
    		seperatePET1(files); // , resultCount
    		SeparatePET2();
    	}
    }
    
    // Parameters.pathOut, Parameters.outFile,
    public  void seperate_SET() throws Exception { // , int[][] resultCount
    
		int numOfRedundent = 0;
    	int next;
		OligoFile[] ofUm = new OligoFile[Parameters.SEQ_VAR.length];
		for (int chromosome = 0; chromosome < Parameters.SEQ_VAR.length; chromosome++) {
			String file = Parameters.ch_name1 + Parameters.SEQ_VAR[chromosome] + Parameters.ch_name2;
			ofUm[chromosome] = new OligoFile(Parameters.pathOut + file + ".um.s.id", Parameters.pathRefSeq + file, OligoFile.W);
		}

		OligoFile[] of = new OligoFile[2];
		of[0] = new OligoFile(Parameters.pathOut + "s.m0.s.id", "",	OligoFile.R);
		of[1] = new OligoFile(Parameters.pathOut + "s.m1.s.id", "",	OligoFile.R);

    	int oldSt = Oligo.sortIndex;
    	Oligo.sortIndex = Oligo.ID;
		PriorityQueue<OligoFileTop> pQueue = new PriorityQueue<OligoFileTop>();
		OligoFileTop oft;

		for (int j = 0; j < of.length; j++) {
			oft = of[j].new OligoFileTop();
			Oligo ol = new Oligo();
			of[j].readOligo(ol);
			oft.ol = ol;
			oft.id = j;
			pQueue.add(oft);
		}

		ArrayList<Oligo> temp = new ArrayList<Oligo>(); //((int) (of[0].numberOfOligos + of[1].numberOfOligos));

		int ID = pQueue.peek().ol.id1;
		while (pQueue.size() > 0) {
			oft = pQueue.poll();
			next = oft.id;
			if (oft.ol.id1 == ID) {
				temp.add(oft.ol);
			} else {
				ID = oft.ol.id1;
				saveUmSET(temp, ofUm, numOfRedundent);
				temp.add(oft.ol);
			}
			Oligo ol = new Oligo();
			if (of[next].readOligo(ol)) {
				oft.ol = ol;
				oft.id = next;
				pQueue.add(oft);
			}
		}

		saveUmSET(temp, ofUm, numOfRedundent);

		System.err.println("numOfRedundent:\t" + numOfRedundent);
		for (int chromosome = 0; chromosome < Parameters.SEQ_VAR.length; chromosome++) {
			ofUm[chromosome].close();
			ofUm[chromosome].writeMeta();
		}
		Oligo.sortIndex = oldSt;
   }   
    
    public  void seperatePET1(String[] files) throws Exception { // , int[][] resultCount
    	int bksz = 0;
    	int totalPETs = 0;
    	int[] hist = new int[Parameters.petHistSize];
    	int dist = 0;
    	for (int i = 0; i < Parameters.petHistSize; i++) {
    		hist[i] = 0;
    	}
		Oligo oligo2;
    	Oligo lastO;
		lastO    = new Oligo();
		lastO.ch = -1;

    	ofMatch    = new OligoFile(Parameters.pathOut + Parameters.outFile + ".m0.s.id", "", OligoFile.R);
    	
		idx = 0;
		clearMeta();
		bunk[DATA].clear();
		ofMatch.readOligo(oligo[DATA]);
		oligo2 = new Oligo(oligo[DATA]);
		addOligo(DATA);
		ofMatch.readOligo(oligo[DATA]);
		while(ofMatch.available() > 0) { 
			while (ofMatch.available() > 0 && oligo[DATA].id1 == oligo2.id1 && oligo[DATA].getSide() == oligo2.getSide()) { 
				addOligo(DATA);
				ofMatch.readOligo(oligo[DATA]); 
			}
			if (ofMatch.available() == 0) { 
				if (oligo[DATA].id1 == oligo2.id1 && oligo[DATA].getSide() == oligo2.getSide()) {
					addOligo(DATA);
				} 
			} else {
				oligo2 = new Oligo(oligo[DATA]);
			}
			
			bksz = bunk[DATA].size();
		    Iterator<Oligo> it = bunk[DATA].iterator();
    		oligo[DATA] = (Oligo) it.next();
			if (bksz == 1 && oligo[DATA].isUM()) { // --------------------------------------------- and um
	    		if(lastO.id1 == oligo[DATA].id1 && lastO.ch == oligo[DATA].ch && lastO.getSide() != oligo[DATA].getSide()) {
	    			dist = Math.abs(lastO.start -oligo[DATA].start);
	    			if (dist < Parameters.petHistSize) {
	    				hist[dist]++;
	    				totalPETs++;
	    			}
	    			lastO.ch = -1;
	    		} else {
	    			lastO = new Oligo(oligo[DATA]);
	    		}
			} 
	        if (ofMatch.available() > 0) {
		        bunk[DATA].clear();
		        oligo[DATA] = new Oligo(oligo2);
		        addOligo(DATA);
	        	ofMatch.readOligo(oligo[DATA]);  
	        }
		}
		ofMatch.close();
    	bunk[DATA].clear();
    	bunk[DATA].trimToSize();

    	PrintWriter out;
		out = new PrintWriter(new BufferedWriter(new FileWriter(new File(Parameters.pathOut + "/petHist"))));
		out.println(totalPETs);
		for (int i = 0; i < Parameters.petHistSize; i++) {
			out.println( i + "\t" + hist[i]);
		}
		out.close();
    }   
    
    public void SeparatePET2() throws Exception {
    	int next;
    	int biggest_num_index = 0;
		int low_index         = 0;
		int high_index        = 0;
		float[] histogram = new float[Parameters.petHistSize];
		PrintWriter outUN  = new PrintWriter(new FileWriter(new File(Parameters.pathOut + "um.nm")));
		PrintWriter outUU = new PrintWriter(new FileWriter(new File(Parameters.pathOut + "um.um")));
		
		BufferedReader br = new BufferedReader(new FileReader(new File(Parameters.pathOut + "petHist")));
		float total_reads = (float) Integer.parseInt(br.readLine());
		for (int j = 0; j < Parameters.petHistSize; j++) {
			String st = br.readLine();
			String[] sub_string = st.split("\t");
			histogram[j] = ((float) Integer.parseInt(sub_string[1])) / total_reads;
			if (histogram[j] > histogram[biggest_num_index])
				biggest_num_index = j;
		}
		br.close();

		for (int j = 0; j < Parameters.petHistSize; j++) {
			if (low_index == 0 && histogram[j] > (histogram[biggest_num_index]/1000)) {
				low_index = j;
			}
			if (high_index == 0 && j > biggest_num_index && histogram[j] < (histogram[biggest_num_index]/1000)) {
				high_index = j;
			}
		}
		System.err.println("biggest_num_index:\t" + biggest_num_index + ",\tbiggest_num:\t" + histogram[biggest_num_index] + 
				",\tlow_index:\t" + low_index + "\thigh_index:\t" + high_index + ",\tRange:\t" + (high_index - low_index));
		// ==================================================================================================================
		//*
		
		OligoFile[] ofUm = new OligoFile[Parameters.SEQ_VAR.length];
		for (int chromosome = 0; chromosome < Parameters.SEQ_VAR.length; chromosome++) {
			String file = Parameters.ch_name1 + Parameters.SEQ_VAR[chromosome] + Parameters.ch_name2;
			  ofUm[chromosome] = new OligoFile(Parameters.pathOut + file + ".um.s.id", Parameters.pathRefSeq + file, OligoFile.W);
		}
		// for (int i = 0; i < Parameters.SEQ_VAR.length; i++) {
		OligoFile[] of = new OligoFile[2];
		of[0] = new OligoFile(Parameters.pathOut + "s.m0.s.id", "",	OligoFile.R);
		of[1] = new OligoFile(Parameters.pathOut + "s.m1.s.id", "",	OligoFile.R);

    	int oldSt = Oligo.sortIndex;
    	Oligo.sortIndex = Oligo.ID;
		PriorityQueue<OligoFileTop> pQueue = new PriorityQueue<OligoFileTop>();
		OligoFileTop oft;

		for (int j = 0; j < of.length; j++) {
			oft = of[j].new OligoFileTop();
			Oligo ol = new Oligo();
			of[j].readOligo(ol);
			oft.ol = ol;
			oft.id = j;
			pQueue.add(oft);
		}

		ArrayList<Oligo> temp = new ArrayList<Oligo>(); //((int) (of[0].numberOfOligos + of[1].numberOfOligos));

		int ID = pQueue.peek().ol.id1;
		while (pQueue.size() > 0) {
			oft = pQueue.poll();
			next = oft.id;
			if (oft.ol.id1 == ID) {
				temp.add(oft.ol);
			} else {
				ID = oft.ol.id1;
				saveUmPET(temp, ofUm, outUN, outUU, histogram, low_index, high_index);
				temp.add(oft.ol);
			}
			Oligo ol = new Oligo();
			if (of[next].readOligo(ol)) {
				oft.ol = ol;
				oft.id = next;
				pQueue.add(oft);
			}
		}

		saveUmPET(temp, ofUm, outUN, outUU, histogram, low_index, high_index);

		
		for (int chromosome = 0; chromosome < Parameters.SEQ_VAR.length; chromosome++) {
			ofUm[chromosome].close();
			ofUm[chromosome].writeMeta();
		}
		outUN.close();
		outUU.close();
		Oligo.sortIndex = oldSt;
    }

	private static void saveUmSET(ArrayList<Oligo> temp, OligoFile[] ofUm, int numberOfRedundent) throws Exception {
		int k = temp.size();
		if (k > 1) {
			if (!temp.get(0).isUM()) {
				temp.clear();
				return;
			}
	    	int oldSt       = Oligo.sortIndex;
	    	Oligo.sortIndex = Oligo.WEIGHT;
			Collections.sort(temp);
			Oligo.sortIndex = oldSt;
			numberOfRedundent++;
		}
		ofUm[temp.get(0).ch].writeOligo(temp.get(0));
		temp.clear();
	}

	private static void saveUmPET(ArrayList<Oligo> temp, OligoFile[] ofUm, PrintWriter outUN, PrintWriter outUU,
			float[] histogram, int low_index, int high_index) throws Exception {
		int gap;
		int ch;
		int k = temp.size();
		String RC0 = "";
		String RC1 = "";
		
		if (k == 1) {
			ofUm[temp.get(0).ch].writeOligo(temp.get(0));
			String RC = "";
			if (temp.get(0).isRC())
				RC = "R";
			else
				RC = "F";
			outUN.write(temp.get(0).ch + "\t" + temp.get(0).start + "\t" + RC + "\n");
			temp.clear();
		} else {
			Collections.sort(temp);
		}
		if ( k > 1) {
			if (k == 2 && temp.get(0).getSide() != temp.get(1).getSide()) {
				
				gap = Math.abs(temp.get(0).start - temp.get(1).start);
				
				if (low_index <= gap && gap <= high_index && temp.get(0).ch == temp.get(1).ch
						&& temp.get(0).isRC() != temp.get(1).isRC()) { 
					// ch = temp.get(0).ch;
					temp.get(0).setPET(true);
					temp.get(1).setPET(true);
					temp.get(0).setUM(true);
					temp.get(1).setUM(true);
					// ofUm[ch].writeOligo(temp.get(0));
					// ofUm[ch].writeOligo(temp.get(1));
					// temp.clear();
				} else {
					temp.get(0).setPET(false);
					temp.get(1).setPET(false);
					temp.get(0).setUM(true);
					temp.get(1).setUM(true);
					if (temp.get(0).isRC())
						RC0 = "R";
					else
						RC0 = "F";
					if (temp.get(1).isRC())
						RC1 = "R";
					else
						RC1 = "F";
					outUU.write(temp.get(0).ch + "\t" + temp.get(0).start + "\t" + RC0 + "\t" + 
							temp.get(1).ch + "\t" + temp.get(1).start + "\t" + RC1 + "\n");
				}
				ofUm[temp.get(0).ch].writeOligo(temp.get(0));
				ofUm[temp.get(1).ch].writeOligo(temp.get(1));
				temp.clear();
			} else { // ------------------------------------------------------------ k > 2
				ArrayList<Oligo> possibleOligos = new ArrayList<Oligo>(k - 1);
				int[] sid = new int[2];
				sid[0] = 0;
				sid[1] = 0;
				for (int q = 0; q < k; q++) {
					sid[temp.get(q).getSide()]++;
				}
				if (sid[0] != 1 && sid[1] != 1) {
					temp.clear();
					return;
				}
				int uqSide;
				int uqSideIdx = 0;
				int uqChr = -1;
				int uqStart = -1;
				if (sid[0] == 1)
					uqSide = 0;
				else
					uqSide = 1;
				for (int q = 0; q < k; q++) {
					if (temp.get(q).getSide() == uqSide) {
						uqSideIdx = q;
						break;
					}
				}
				int distance = 0;
				uqChr = temp.get(uqSideIdx).ch;
				uqStart = temp.get(uqSideIdx).start;
				for (int j = 0; j < k; j++) {
					if ( j != uqSideIdx)
						if (temp.get(j).ch == uqChr) {
							distance = Math.abs(temp.get(j).start - uqStart);
							if (distance >= low_index && distance <= high_index)
								possibleOligos.add(temp.get(j));
						}
				}
				
				if (possibleOligos.size() == 0) {
					
					temp.get(uqSideIdx).setPET(false);
					ofUm[uqChr].writeOligo(temp.get(uqSideIdx));
					
				} else if (possibleOligos.size() == 1) {
					
					possibleOligos.get(0).setPET(true);
					temp.get(uqSideIdx).setPET(true);
					if(uqSide == 0)
						ofUm[uqChr].writeOligo(temp.get(uqSideIdx));
					ofUm[uqChr].writeOligo(possibleOligos.get(0));
					if(uqSide == 1)
						ofUm[uqChr].writeOligo(temp.get(uqSideIdx));
					
				} else if (possibleOligos.size() > 1) {
					
					float[] probabilities = new float[possibleOligos.size()];
					for ( int j = 0; j < possibleOligos.size(); j++) {
						distance = Math.abs(possibleOligos.get(j).start - uqStart);
						float fraction = histogram[distance];
						float rank = fraction * possibleOligos.get(j).weight;
						probabilities[j] = rank;
					}
					int highest_index = 0;
					for (int j = 0; j < probabilities.length; j++) {
						if (probabilities[j] > probabilities[highest_index]) {
							highest_index = j;
						}
					}
					possibleOligos.get(highest_index).setPET(true);
					temp.get(uqSideIdx).setPET(true);
					if(uqSide == 0)
						ofUm[uqChr].writeOligo(temp.get(uqSideIdx));
					ofUm[uqChr].writeOligo(possibleOligos.get(highest_index));
					if(uqSide == 1)
						ofUm[uqChr].writeOligo(temp.get(uqSideIdx));
				}
				temp.clear();
			}
		}
	}
    
    public  void generateCoverage(String path, String dt, String pathSeq, String seqName, int uComp) throws Exception {
    	int wkCounter = 0;
    	byte l = 0;
    	int jrc;
    	long idxCv = 0;
    	int  ibp = -1;
    	ofOne    = new OligoFile(path + seqName + ".um.s.id", "", OligoFile.R); // read to DATA
    	getMeta(ofOne);
		openPrbIn(Parameters.pathData + dt + ".prb");
		l = read_Prb(-1, -1);
		int jj;
		int countMMP  = 0;
		int lastID    = 0;
		int countIDMP = 0;
		float olScore;
		while(ofOne.available() > 0) { 
			idxCv++;
			ofOne.readOligo(oligo[DATA]);
			olScore = oligo[DATA].getMappingScore(map.sq0[Parameters.F][oligo[DATA].start], map.sq0[Parameters.R][oligo[DATA].start + oligo[DATA].size - 1], oligo[DATA].weight);
			if (olScore >= 0) {
				if (oligo[DATA].id1 != lastID) {
					if (countIDMP > 1) {
						countMMP++;
					}
					countIDMP = 0;
					lastID = oligo[DATA].id1;
				}
				if (oligo[DATA].isMP()) {
					countIDMP++;
				}
				while(!(oligo[DATA].id1 == seq.id && oligo[DATA].getSide() == l)) {
					l = read_Prb(oligo[DATA].id1, oligo[DATA].getSide()); 
					// System.err.print("> " + oligo[DATA].id1 + " == " + seq.id + "\t&&\t" + oligo[DATA].getSide() + " == " + l);
					if (oligo[DATA].id1 == seq.id && oligo[DATA].getSide() == l) {
						// System.err.println(" --- True");
						break;
					}
					if (oligo[DATA].id1 > seq.id) {
						System.err.println("\t<prb:\t" + seq.id + ", " + l + ">\t,Target:\t" + oligo[DATA].id1 + "\t" + oligo[DATA].getSide());
						Thread.sleep(2000);
					}
				}
				// System.err.println("\nOut");
				for (int j = oligo[DATA].start; j != oligo[DATA].start + oligo[DATA].size; j++) { 
					jj = j - oligo[DATA].start;

					if (oligo[DATA].isRC()) { 
						jrc = oligo[DATA].size - jj - 1;
						if (jrc == oligo[DATA].u0) {
							map.cnt[1][j]++;
							map.snpSc[1][j] += olScore;
							for (int pp = 0; pp < 4; pp++) {
								ibp = comp[pp];
								map.Pb[0][pp][j] += seq.p[jrc][ibp];
							}
							if (Parameters.snpRef) {
								if(map.snpf[j]) {
									map.scoreDistrebution[1][(int) (olScore * (Parameters.rangeScoreDist - 1))]++;
								} else {
									map.scoreDistrebution[0][(int) (olScore * (Parameters.rangeScoreDist - 1))]++;										
								}
							} 
						} else {
							if (!seq.isCrisp(jrc)) {
								map.cnt[2][j]++;
								for (int pp = 0; pp < 4; pp++) {
									ibp = comp[pp];
									map.Pb[1][pp][j] += seq.p[jrc][ibp];
								}
							} else {
								map.snpSc[0][j] += olScore;
								map.cnt[0][j]++;
							}
						}
					} else { 
						if (jj == oligo[DATA].u0) {
							map.cnt[1][j]++;
							map.snpSc[1][j] += olScore;
							for (int pp = 0; pp < 4; pp++) {
								map.Pb[0][pp][j] += seq.p[jj][pp];
							}
							if (Parameters.snpRef) {
								if(map.snpf[j]) {
									map.scoreDistrebution[1][(int) (olScore * (Parameters.rangeScoreDist - 1))]++;
								} else {
									map.scoreDistrebution[0][(int) (olScore * (Parameters.rangeScoreDist - 1))]++;										
								}
							} 
						} else { 
							if (!seq.isCrisp(jj)) {
								map.cnt[2][j]++;
								for (int pp = 0; pp < 4; pp++) {
									map.Pb[1][pp][j] += seq.p[jj][pp];
								}
							} else {
								map.snpSc[0][j] += olScore;
								map.cnt[0][j]++;									
							}
						}
					}

				}
			} else {
				wkCounter++;
			}
		}
		System.err.println("\n -- WK Counter" + ux + ":\t" + wkCounter);
		ofOne.close();
		if (ux > 0) {
			closePrbIn();
		} 
		else { 
			closePrbIn();
		}
		float sum;
		for (int i = 0; i < map.size; i++) { 
			sum = 0;
			for (int ll = 0; ll < 4; ll++) {
				sum += map.Pb[0][ll][i];
			}
			if (sum > 0)
				for (int ll = 0; ll < 4; ll++) {
					map.Pb[0][ll][i] = 100 * map.Pb[0][ll][i] / sum;
				}
			sum = 0;
			for (int ll = 0; ll < 4; ll++) {
				sum += map.Pb[1][ll][i];
			}
			if (sum > 0)
				for (int ll = 0; ll < 4; ll++) {
					map.Pb[1][ll][i] = 100 * map.Pb[1][ll][i] / sum;
				}
		}
    }
    
    public  void findSNPs(String path, String file) throws Exception {
		System.gc();
    	int sid;
    	long[] xx = new long[4];
    	long[] ax = new long[4];
    	// long aa;
    	int x;
    	int z;
    	String fl = path + "ScoreResults/" + file;
    	int countBoth = 0;

		//*		
    	float[] average = new float[2];
    	float[] var     = new float[2];
    	// float[] max     = new float[2];
    	float[] cut     = new float[2];
    	float tmp;
		for (int j = 0; j < 2; j++) {
			var[j] = 
			// max[j] = 
			average[j] = 0;
		}

		for (int i = 0; i < map.size; i++) {
			average[0] += map.cnt[0][i] + map.cnt[1][i] + map.cnt[2][i];
    	}
    	for (int j = 0; j < 2; j++) {
    		average[j] = average[j] / map.size;
    	}
    	//*
    	for (int i = 0; i < map.size; i++) {
    		tmp = map.cnt[0][i] + map.cnt[1][i] + map.cnt[2][i] - average[0];
    		var[0] += (float) Math.sqrt(tmp * tmp);
    	}
		for (int j = 0; j < 1; j++) { 
			var[j] = var[j] / map.size;
		}
    	if (average[0] + average[1] < 2) {
    		cut[0] = (float) 0.4;
    	} else {
    		cut[0] = average[0] - var[0];
    		if (cut[0] < 0.5)
    			cut[0] = (float) 0.5;
    	}
    	cut[0] = cut[0] / 5;
    	//*/
    	ax[0] = ax[1] = xx[0] = xx[1] = xx[2] = 0;
    	for (int i = 0; i < map.size - Parameters.oMxSize; i++) {
    		if ((map.cnt[0][i] + map.cnt[1][i] + map.cnt[2][i]) > 0) {
    			xx[3]++;
    		}
    		for (int l = 0; l < 3; l++) {
    			if (map.cnt[l][i] > 0) {
    				xx[l]++;
    				ax[l] += map.cnt[l][i];
    			}
    		}
    	}
    	//*	
    	// reading sequencebility data into map
    	String sapFl = Parameters.pathDB + Parameters.DATABASE + file + ".map." + (Parameters.oMxSize + Parameters.mapAdjust); 
    	FileInputStream 	fis    = new FileInputStream(sapFl);
    	BufferedInputStream bis    = new BufferedInputStream(fis, 127000);
    	DataInputStream		strIns = new DataInputStream(bis);
		for (int i = 0; i < map.size - Parameters.oMxSize; i++) {
			map.sq0[Parameters.F][i] = strIns.readByte();
		}
		strIns.close();
		bis.close();
		fis.close();
		//*/
		System.err.println(fl + ".snp");
		BufferedWriter writer = new BufferedWriter(new FileWriter(fl + ".snp"));
    	writer.write("total  coverage:\t(\t" + ax[0] + "\t+\t" + ax[1] + "\t+\t" + ax[2] + "\t) =\t" + (ax[0] + ax[1] + ax[2]) + "\tX =\t" + Parameters.trim((float) (ax[0] + ax[1] + ax[2]) / (float) (map.size - Parameters.oMxSize)) + "\n");
    	writer.write("actual coverage:\t(\t" + xx[0] + "\t+\t" + xx[1] + "\t+\t" + xx[2] + "\t) =\t" + (xx[0] + xx[1] + xx[2]) + "\tX =\t" + Parameters.trim((float) (xx[3]) / (float) (map.size - Parameters.oMxSize)) + "\n");
    	writer.write("cut:\t" + cut[0] + "\tAverage:\t" + average[0] + "\tvar:\t" + var[0] + "\n");
    	if (Parameters.snpRef) 
    		writer.write("\t");
    	writer.write("SNPid\tLocation\tmap\tcmCount\tmCount\tnmCount\twSNP%\tcSNP%\tncSNP%\tcmScore\tnmScore1\tRef\tMPB\tPm(A)\tPm(C)\tPm(G)\tPm(T)\tPnm(A)\tPnm(C)\tPnm(G)\tPnm(T)\n");
    	if (Parameters.snpRef) 
    		writer.write("\t");
    	writer.write("-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\n");
    	sid = 0;
    	float wsnpp;
    	for (int i = 0; i < map.size; i++) {
    		x = getMPBP(i, 0);
    		wsnpp = ((((float) map.snpSc[1][i]) * map.Pb[0][x][i] / 100) / ((float) map.snpSc[0][i] + map.snpSc[1][i]));
    		if( map.cnt[1][i] >= Parameters.snpMinCover   && 
    			map.snpSc[1][i] > Parameters.snpMinWeight &&
    			wsnpp > Parameters.snpPerW                &&
    			((float) map.cnt[1][i]) / ((float) map.cnt[0][i] + map.cnt[1][i]) > Parameters.snpPer) // ---- (cut[1] + cut[0])) 
    		{ 
    			z = getBpInt(map.charSeq[i]); 
    			if(z < 0) {
    				System.err.println("z:\t" + z + "\tchar\t" + map.charSeq[i]);
    			}
    			x = getMPBP(i, 0);
    			//*
    			if (x == z)
    				x = npbp;
    			//*/
    			// pz = getBPP(i, z);
    			// px = getBPP(i, x);
    			// if((pz < (Parameters.pQuality * 0.02 * map.cnt[1][i]) && px > (90.0 - 5 * map.cnt[1][i]))) 
    			{ // && pz < 0.1
    				if (Parameters.snpRef) { 
						if (map.snpf[i]) {
							countBoth++;
							writer.write("T\t");
						} else {
							writer.write("F\t");
						}
    				} // ((float) map.snpSc[1][i]) / ((float) map.snpSc[0][i] + map.snpSc[1][i])
    				writer.write(sid + "\t" + (i+1) + "\t" + map.sq0[Parameters.F][i] + "\t" + map.cnt[0][i] + "\t" + map.cnt[2][i] + "\t" + map.cnt[1][i] + "\t");
    				writer.write(Parameters.trim(100 * wsnpp) + "%\t");
    				writer.write(Parameters.trim(100 * ((float) map.cnt[1][i]) / ((float) map.cnt[0][i] + map.cnt[1][i])) + "%\t");
    				writer.write(Parameters.trim(100 * ((float) map.Pb[1][x][i]) / ((float) map.Pb[1][getBpInt(map.charSeq[i])][i] + map.Pb[1][x][i])) + "%\t");
    				writer.write(Parameters.trim(map.snpSc[0][i]) + "\t" + Parameters.trim(map.snpSc[1][i]) + "\t" + map.charSeq[i] + "\t" + nucl[x]);
    				sid++;
    				for (int j = 0; j < 4; j++) {
    					writer.write("\t" + Parameters.trim(map.Pb[1][j][i]));
    				}
    				for (int j = 0; j < 4; j++) {
    					writer.write("\t" + Parameters.trim(map.Pb[0][j][i]));
    				}
    				writer.write("\n");
    			}
    		}
    	}
    	//
    	if (Parameters.snpRef) {
    		long[] countSNPs = new long[3];
    		countSNPs[0] = countSNPs[1] = 0;
			writer.write("\ncountBoth\t" + countBoth + "\t" + sid + "\n\n");		
    		for (int i = 0; i < Parameters.rangeScoreDist; i++) {
    			countSNPs[0] += map.scoreDistrebution[0][i];
    			countSNPs[1] += map.scoreDistrebution[1][i];
    			writer.write(i + "\t" + map.scoreDistrebution[0][i] + "\t" +  map.scoreDistrebution[1][i] + "\n");
    		}
    		writer.write("-----\t-----\n" + countSNPs[0] + "\t" + countSNPs[1] + "\n");
    	}
    	writer.close();
    	
    	// map.scoreDistrebution[1][(int) (olScore * Parameters.rangeScoreDist / 3)]++;
    	BufferedWriter writer0 = new BufferedWriter(new FileWriter(fl + ".cover"));
    	// writer0.write(ax[0] + "\t" + ax[1] + "\t" + ax[2] + "\t" + (map.size - Parameters.oMxSize) + "\n");
    	// writer0.write(xx[0] + "\t" + xx[1] + "\t" + xx[2] + "\n");
    	for (int i = 0; i < map.size - Parameters.oMxSize; i++) {
    		for (int l = 0; l < 3; l++) {
    			writer0.write("\t" + map.cnt[l][i]);
    		}
    		writer0.write("\n");
    	}
    	writer0.close();
    	
    	String mapFl = Parameters.pathDB + Parameters.DATABASE + file + ".map." + (Parameters.oMxSize + Parameters.mapAdjust); 
    	int cnvSize = (map.size / Parameters.CNVcomp) + 1;
    	float[] cnv = new float[cnvSize];
    	float   cnvAvrg = 0;
    	float[] mnv = new float[cnvSize];
    	float   mnvAvrg = 0;
    	for (int i = 0; i < cnvSize; i++) {
    		cnv[i] = mnv[i] = 0;
    	}
    	System.err.println(mapFl);
    	// 	String mapFl = Parameters.pathDB + Parameters.DATABASE + file + ".map." + Parameters.oMxSize;
    	FileInputStream 	fi    = new FileInputStream(mapFl);
    	BufferedInputStream bi    = new BufferedInputStream(fi, 127000);
    	DataInputStream		strIn = new DataInputStream(bi);

		for (int i = 0; i < map.size - Parameters.oMxSize; i++) {
			mnv[i/Parameters.CNVcomp] += strIn.readByte();
			cnv[i/Parameters.CNVcomp] += map.cnt[0][i] + map.cnt[1][i] + map.cnt[2][i];
		}
		
		strIn.close();
		bi.close();
		fi.close();

		for(int i = 0; i < cnv.length; i++) {
			cnvAvrg += cnv[i];
			mnvAvrg += mnv[i];
		}
		cnvAvrg = cnvAvrg / cnv.length;
		mnvAvrg = mnvAvrg / cnv.length;
		
    	BufferedWriter writer1 = new BufferedWriter(new FileWriter(fl + ".cnv"));
		for(int i = 0; i < cnv.length; i++)
			writer1.write(i + "\t" + Parameters.trim((cnv[i]/cnvAvrg) * 100) + "\t" + Parameters.trim((mnv[i]/mnvAvrg) * 100) + "\n");
		writer1.close();

    	map.ceil();

    	System.err.println("map.size - oSize\t" + (map.size - Parameters.oMxSize));
    	map = null;
    	System.runFinalization();
		System.gc();
    }
    
    public  void compareSNPs1(String path, String file, String multiPath) throws Exception {
		System.gc();
    	int sid;
    	long[] xx = new long[4];
    	long[] ax = new long[4];
    	int x;
    	int z;
    	String fl = multiPath + "/" + file; // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    	ax[0] = ax[1] = xx[0] = xx[1] = xx[2] = 0;
    	for (int i = 0; i < map.size - Parameters.oMxSize; i++) {
    		if ((map.cnt[0][i] + map.cnt[1][i] + map.cnt[2][i]) > 0) {
    			xx[3]++;
    		}
    		for (int l = 0; l < 3; l++) {
    			if (map.cnt[l][i] > 0) {
    				xx[l]++;
    				ax[l] += map.cnt[l][i];
    			}
    		}
    	}
    	String sapFl = Parameters.pathDB + Parameters.DATABASE + file + ".map." + (Parameters.oMxSize + Parameters.mapAdjust); 
    	FileInputStream 	fis    = new FileInputStream(sapFl);
    	BufferedInputStream bis    = new BufferedInputStream(fis, 127000);
    	DataInputStream		strIns = new DataInputStream(bis);
		for (int i = 0; i < map.size - Parameters.oMxSize; i++) {
			map.sq0[Parameters.F][i] = strIns.readByte();
		}
		strIns.close();
		bis.close();
		fis.close();

		System.err.println(fl + ".snp");
		BufferedWriter writer = new BufferedWriter(new FileWriter(fl + ".snp"));
    	writer.write("total  coverage:\t(\t" + ax[0] + "\t+\t" + ax[1] + "\t+\t" + ax[2] + "\t) =\t" + (ax[0] + ax[1] + ax[2]) + "\tX =\t" + Parameters.trim((float) (ax[0] + ax[1] + ax[2]) / (float) (map.size - Parameters.oMxSize)) + "\n");
    	writer.write("actual coverage:\t(\t" + xx[0] + "\t+\t" + xx[1] + "\t+\t" + xx[2] + "\t) =\t" + (xx[0] + xx[1] + xx[2]) + "\tX =\t" + Parameters.trim((float) (xx[3]) / (float) (map.size - Parameters.oMxSize)) + "\n");
    	writer.write("Location\tmap\tcmCount\tmCount\tnmCount\twSNP%\tcSNP%\tncSNP%\tRef\tMPB\n");
    	writer.write("-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\t-----\n");
    	sid = 0;
    	float wsnpp;
    	for (int i = 0; i < map.size; i++) {
    		x = getMPBP(i, 0);
    		wsnpp = ((((float) map.snpSc[1][i]) * map.Pb[0][x][i] / 100) / ((float) map.snpSc[0][i] + map.snpSc[1][i]));
			z = getBpInt(map.charSeq[i]); 
			if(z < 0) {
				System.err.println("z:\t" + z + "\tchar\t" + map.charSeq[i]);
			}
			x = getMPBP(i, 0);

			if (x == z)
				x = npbp;
			writer.write((i+1) + "\t" + map.sq0[Parameters.F][i] + "\t" + map.cnt[0][i] + "\t" + map.cnt[2][i] + "\t" + map.cnt[1][i] + "\t");
			writer.write(Parameters.trim(100 * wsnpp) + "\t");
			writer.write(Parameters.trim(100 * ((float) map.cnt[1][i]) / ((float) map.cnt[0][i] + map.cnt[1][i])) + "\t");
			writer.write(Parameters.trim(100 * ((float) map.Pb[1][x][i]) / ((float) map.Pb[1][getBpInt(map.charSeq[i])][i] + map.Pb[1][x][i])) + "\t");
			writer.write(map.charSeq[i] + "\t" + nucl[x]);
			sid++;
			writer.write("\n");   		
    	}
    	writer.close();
    	System.runFinalization();
		System.gc();
    } // --------------------------------------------------------------------------------------- compareSNPs1

    private int  getAddress() throws Exception {
    	int x = 10000000;
    	String line = reader.readLine(); 
		if (line != null) {
			StringTokenizer tokens = new StringTokenizer(line);
			tokens.nextToken();
			x = Integer.parseInt(tokens.nextToken(), 10);
		} else {
			x = -1;
		}
		x--;
    	return x;
    }

	private class Sequencability {
		byte[] C_map;
		public Sequencability(int size) {
			C_map = new byte[size];
    		for (int j = 0; j < size; j++) {
    			C_map[j] = 0;
    		}
 		}
	}

    public  void sequencability(String fdb, String[] seqNames, int[] seqSize, int sz) throws Exception {
    	long ofidx = 0;
    	Sequencability[] sq = new Sequencability[seqNames.length];
    	for (int i = 0; i < seqNames.length; i++) {
			// System.err.println("Chr:\t" + (i+1));
    		sq[i] = new Sequencability(seqSize[i]);
    	}
    	// short[][] C_map;
    	if (display == true)
    		System.err.println("sequencability:\t" + fdb + "\tSize:\t" + sz);
    	// int        x = 2;
    	long[] histo   = new long[2 * sz + 1];
		for (int j = 0; j < histo.length; j++)
			histo[j] = 0;
		// System.err.println("Will be Writing map to:\t" + fdb + seqNames[0] + ".map." + sz);
		ofidx = 0;
    	OligoFile ofile = new OligoFile(fdb, "", OligoFile.R);
  		while(ofile.readOligo(oligo[DATA])) { // ------------------------------------------------------
   			ofidx++;
   			if (ofidx % 100000000 == 0)
   				System.err.println("\t" + (ofidx/100000000) + "00,000,000");
   			//*
			if (oligo[DATA].isRC()) {
				if(oligo[DATA].u0 < sz) {
					for (int i = 0; i < sz; i++) {
						sq[oligo[DATA].ch].C_map[oligo[DATA].start + Parameters.dbOligoSize - i - 1] += (byte) (oligo[DATA].weight + 0.2);
					}
				}								
			} else {
				if(oligo[DATA].u0 < sz) {
					for (int i = 0; i < sz; i++) {
						sq[oligo[DATA].ch].C_map[oligo[DATA].start + i] += (byte) (oligo[DATA].weight + 0.2);
					}
				}				
			}
			//*/
		}		
		for (int ifile = 0; ifile < seqNames.length; ifile++) {
			for (int j = 0; j < seqSize[ifile]; j++) {
				if (sq[ifile].C_map[j] > ( 2 * sz)) {
					System.err.println("i0\t" + sq[ifile].C_map[j] + ",\tchr\t" + ifile + "\tidx\t" + j);
				} else {
					histo[sq[ifile].C_map[j]]++;
				}
			}
		}
		
		ofile.close();	
		// ----------------------------------------------------------------------- Writing Results
		for (int ifile = 0; ifile < seqNames.length; ifile++) {
			// System.err.println("Writing map to:\t" + fdb + seqNames[ifile] + ".map." + sz);
			FileOutputStream     fwOne = new FileOutputStream(fdb + seqNames[ifile] + ".map." + sz);
			BufferedOutputStream bwOne = new BufferedOutputStream(fwOne);
			DataOutputStream     file  = new DataOutputStream(bwOne);			
			for (int i = 0; i < seqSize[ifile]; i++) {
				file.writeByte(sq[ifile].C_map[i]);
			}
		    file.close();
		    bwOne.close();
		    fwOne.close();
	 	}
		sq = null;
		System.gc();
		/*
		System.out.println("sequencability\t# of Noclutides");
		for (int i = 0; i < histo.length; i++) {
			System.out.println("\t" + i + "\t" + histo[i]);
		}
		//*/
    }

    public  void sequencability0(String fdb, String[] seqNames, int[] seqSize) throws Exception {
    	long ofidx = 0;
    	Sequencability[][] sq = new Sequencability[seqNames.length][2];
    	for (int i = 0; i < seqNames.length; i++) {
			// System.err.println("Chr:\t" + (i+1));
    		sq[i][Parameters.F] = new Sequencability(seqSize[i]);
    		sq[i][Parameters.R] = new Sequencability(seqSize[i]);
    	}
		ofidx = 0;
    	OligoFile ofile = new OligoFile(fdb, "", OligoFile.R);
  		while(ofile.readOligo(oligo[DATA])) { // ------------------------------------------------------
   			ofidx++;
   			if (ofidx % 100000000 == 0)
   				System.err.println("\t" + (ofidx/100000000) + "00,000,000");
   			//*
			if (oligo[DATA].isRC()) {
				// System.err.println(oligo[DATA].start + Parameters.dbOligoSize + "\t" + sq[oligo[DATA].ch][Parameters.R].C_map.length);
				if(sq[oligo[DATA].ch][Parameters.R].C_map[oligo[DATA].start + Parameters.dbOligoSize - 1] < oligo[DATA].u0) {
					sq[oligo[DATA].ch][Parameters.R].C_map[oligo[DATA].start + Parameters.dbOligoSize - 1] = oligo[DATA].u0;
				}								
			} else {
				if(sq[oligo[DATA].ch][Parameters.F].C_map[oligo[DATA].start] < oligo[DATA].u0) {
					sq[oligo[DATA].ch][Parameters.F].C_map[oligo[DATA].start] = oligo[DATA].u0;
				}				
			}
			//*/
		}		
		ofile.close();	
		// ----------------------------------------------------------------------- Writing Results
		for (int ifile = 0; ifile < seqNames.length; ifile++) {
			FileOutputStream     fwOne = new FileOutputStream(fdb + seqNames[ifile] + ".u0");
			BufferedOutputStream bwOne = new BufferedOutputStream(fwOne);
			DataOutputStream     file  = new DataOutputStream(bwOne);			
			for (int i = 0; i < seqSize[ifile]; i++) {
				file.writeByte(sq[ifile][Parameters.F].C_map[i]);
				file.writeByte(sq[ifile][Parameters.R].C_map[i]);
			}
		    file.close();
		    bwOne.close();
		    fwOne.close();
	 	}
		sq = null;
		System.gc();
    }

    public  void filterRedundant(String fin, String fout) throws Exception {
    	if (display == true)
    		System.err.println("\n -------- sortFilterRedundant " + fin);
    	Oligo oligo2 = new Oligo();
    	char bp0;
    	ofOne    = new OligoFile(fin, "", OligoFile.R);
    	ofTwo    = new OligoFile(fout, "", OligoFile.W);
		readMeta(ofOne);     // ----------------------------------------------------------
		// ofile = new OligoFile(fin, OligoFile.R);
		clearMeta();
		idx = 0;
		bunk[0].clear();
		oligo2.id1 = -100;
		while(ofOne.available() > 0) { 
			ofOne.readOligo(oligo[DATA]); 
			if (oligo[DATA].id1 != oligo2.id1) {
				if (oligo[DATA].isUQ())
					numberOfUQ++;
				else 
					numberOfUD++;
				bp0 = oligo[DATA].getBPat(0);
				if (bp0 == 'A' || bp0 == 'T') {
					weightOfAT += oligo[DATA].weight;
				} else {
					weightOfCG += oligo[DATA].weight;
				}
				ofTwo.writeOligo(oligo[DATA]);
				idx++;
				oligo2 = new Oligo(oligo[DATA]);
				numberOfOligos++;
			}
		}
		ofOne.close(); 
		ofTwo.close();
	    // System.out.println("From file " + fin + " removed oligos \t" + (oldNumberOfOligos - numberOfOligos));
	    if (display == true)
	    	System.err.println(" -------- sortFilterRedundant " + fin + " End\n");
	
    }

    public int split(String line) {
    	// StringTokenizer tokens1 = new StringTokenizer();
		int x;
		int sz = 0;
    	StringTokenizer tokens = new StringTokenizer(line);
    	int lineSize = tokens.countTokens();
    	// System.err.print("Prb size:\t" + lineSize + "\t");
		if (Parameters.ETag == Parameters.PET) {
	    	int rdMxSz =  (lineSize/4)/2 - Parameters.skipCycles;
	    	// System.err.print("rdMxSz:\t" + rdMxSz + "\t");
			if (rdMxSz > Parameters.oMxSize)
				sz = Parameters.oMxSize;
			else
				sz = rdMxSz;
			// System.err.println("sz:\t" + sz + "\tskipCycles:\t" + Parameters.skipCycles + "\t" + (Parameters.skipCycles + rdMxSz - sz));
			for (int sk = 0; sk < Parameters.skipCycles; sk++) { 			// ------------------------------------------------ skip
		    	for (int j = 0; j < 4; j++) {
		    		tokens.nextToken();
		    	}
			}
	    	for (int i = 0; i < sz; i++) {
	    		for (int j = 0; j < 4; j++) {
	       			// x = Integer.parseInt(tokens.nextToken(), 10);
	       			seq.q[0][i][j] = (byte) Integer.parseInt(tokens.nextToken(), 10);
	    		}
	    	}    	   	
	    	// System.err.println("Parameters.skipCycles:\t" + Parameters.skipCycles + "\trdMxSz:\t" + rdMxSz + "\tsz:\t" + sz);
	    	// System.err.println("Parameters.skipCycles + rdMxSz - sz:\t" + (Parameters.skipCycles + rdMxSz - sz));
	    	for (int sk = 0; sk < (Parameters.skipCycles + rdMxSz - sz); sk++) { 			// ------------------------------------------------ skip
	    		for (int j = 0; j < 4; j++) {
	       			tokens.nextToken();
	    		}
	    	}    	   	
	    	for (int i = 0; i < sz; i++) {
	    		for (int j = 0; j < 4; j++) {
	    			// x = new Integer(tokens.nextToken());
	    			seq.q[1][i][j] = (byte) Integer.parseInt(tokens.nextToken(), 10);
	    		}
	    	}    	  
		} else {
	    	sz =  tokens.countTokens();
	    	// System.err.println("sz:\t" + sz);
			if (sz > Parameters.oMxSize)
				sz = Parameters.oMxSize;
	    	for (int i = 0; i < sz; i++) {
	    		for (int j = 0; j < 4; j++) {
	       			x = Integer.parseInt(tokens.nextToken(), 10);
	       			seq.q[0][i][j] = (byte) x;
	    		}
	    	}    	   				
		}
    	return sz;
    }
 
    private void getMeta(OligoFile of) {
    	numberOfOligos = of.numberOfOligos;
    	weightOfAT     = of.weightOfAT;
    	weightOfCG     = of.weightOfCG;
    	numberOfUQ     = of.numberOfUQ;
    	numberOfUD     = of.numberOfUD;
    }
    private void readMeta(OligoFile of) throws Exception {
		numberOfOligos = of.numberOfOligos;
		numberOfUQ     = of.numberOfUQ; 
		numberOfUD     = of.numberOfUD; 
		weightOfAT     = of.weightOfAT;
		weightOfCG     = of.weightOfCG;
		if (display == true) {
			System.out.println(
					"Number of Us ...:\t" + "\n" +
					"Number of Oligos:\t" + numberOfOligos + "\n" + 
					"Number of UQ ...:\t" + numberOfUQ + "\n" + 
					"Number of UD ...:\t" + numberOfUD + "\n" + 
					"Weight of Oligos:\t" + Parameters.trim(weightOfAT  + weightOfCG) + "\n" + 
					"Weight of AT ...:\t" + Parameters.trim(weightOfAT) + "\n" + 
					"Weight of CG ...:\t" + Parameters.trim(weightOfCG) + "\n" +
					"Ratio AT/CG ....:\t" + ((float) weightOfAT / (float) weightOfCG));
		}
    }  
    private void clearMeta() {
    	numberOfOligos = 0;
    	weightOfAT     = 0;
    	weightOfCG     = 0;
    	numberOfUQ     = 0;
    	numberOfUD     = 0;
    }
    public  void fillCode() { // to fill the matrix for coding string to long
		code        = new long[4][Parameters.dbOligoSize];
		Oligo.code  = new long[4][Parameters.dbOligoSize];
		for (int bpIdx = 0; bpIdx < 4; bpIdx++) {
		    code[bpIdx][Parameters.dbOligoSize - 2] = bpIdx;
		    code[bpIdx][Parameters.dbOligoSize - 2] = code[bpIdx][Parameters.dbOligoSize - 2] << (Parameters.dbOligoSize-2);
		    for (int i = Parameters.dbOligoSize - 4; i >= 0; i -=2) {
			    code[bpIdx][i] = code[bpIdx][i+2] >> 2;
		    }
		    for (int i = 0; i < Parameters.dbOligoSize/2; i++) {
			    code[bpIdx][i] = code[bpIdx][2*i];
		    }		
		}
		for (int bpIdx = 0; bpIdx < 4; bpIdx++) {
			for (int i = 0; i < Parameters.dbOligoSize; i++) {
				Oligo.code[bpIdx][i] = code[bpIdx][i];
			}
		}
		// displayCodeMatrix();
	}
/*
	public  void getFilesSize(String dbpath, String[] fl, int[] fSizes) throws Exception {
		char c;
		
		for (int i = 0; i < fl.length; i++) {
			fSizes[i] = 0;
			FileReader chFile  = new FileReader(dbpath + fl[i]);
			c = (char) chFile.read();
			while(c != 10) {
				c = (char) chFile.read();
			}
			while(chFile.ready()) {
				c = (char) chFile.read();
				if (c != 10 && c != 13) {
					fSizes[i]++;
				}
			}
			chFile.close();
			System.err.println(fl[i] + "\t" + fSizes[i]);			
		}
	} //*/
	
	// rs.createMap(<<Parameters.pathRefSeq + files[i]>>, <<files[i]>>, i);
	public  void createMap(String fl, String outf, int fInt) throws Exception {
		// System.err.println("createMap\t" + fl + "\t" + fInt + "\t" + refSeqSize[fInt]);
		map = new Map(refSeqSize[fInt], fl, outf);
		// System.err.println("createMap\t" + fl + "\t" + fInt + "\t" + refSeqSize[fInt]);
	}

	/*
    public void fillU(String fl) throws Exception {
		System.err.println(" fillU " + fl);
    	Oligo oligo0;
    	// int u0 = 0;
    	byte u1;
    	byte[] l = new byte[2];
    	openStreamIn(fl + ".db.s.sq", DATA);
		readMeta(DATA, fl + ".db.s.sq");
		idx = 0;
		bunk[0].clear();
		while(strIn[DATA].available() > 0) {
			readOligoEx(DATA);
			addOligo(0);
			idx++;
		}
		
		closeStreamIn(DATA);
	    // int k = 1;
		if (Oligo.nou > 1)
	    for ( int u = 0; u < oligoSize; u++) {
	    	System.err.println("itation:\t" + u + "/" + (oligoSize-1));
	    	Oligo.clearMask();
	    	Oligo.setMask(u);
	    	Collections.sort(bunk[0]); 
	    	Iterator it = bunk[0].iterator();
	    	l[1] = 0;
	    	oligo0 = (Oligo) it.next ();
	    	for (; it.hasNext(); ) {
	    		oligo[DATA] = (Oligo) it.next ();
	    		l[0] = l[1];
	    		l[1] = oligo[DATA].match(oligo0);
	    		if (l[0] > l[1]) {
	    			u1 = l[0];
	    		} else {
	    			u1 = l[1];
	    		}
	    		if (u1 > oligo0.u[1])
	    			oligo0.u[1] = u1;
	    		oligo0 = oligo[DATA];
	    	}
	    	if (Oligo.nou > 2) {
	    	    for ( int u2 = u+1; u2 < oligoSize; u2++) {
	    	    	Oligo.clearMask();
	    	    	Oligo.setMask(u);
	    	    	Oligo.setMask(u2);
	    	    	Collections.sort(bunk[0]); 
	    	    	Iterator it2 = bunk[0].iterator();
	    	    	l[1] = 0;
	    	    	oligo0 = (Oligo) it2.next ();
	    	    	for (; it2.hasNext(); ) {
	    	    		oligo[DATA] = (Oligo) it2.next ();
	    	    		l[0] = l[1];
	    	    		l[1] = oligo[DATA].match(oligo0);
	    	    		if (l[0] > l[1]) {
	    	    			u1 = l[0];
	    	    		} else {
	    	    			u1 = l[1];
	    	    		}
	    	    		if (u1 > oligo0.u[2])
	    	    			oligo0.u[2] = u1;
	    	    		oligo0 = oligo[DATA];
	    	    	}
	    	    	if (Oligo.nou > 3) {
	    	    	    for ( int u3 = u+1; u3 < oligoSize; u3++) {
	    	    	    	Oligo.clearMask();
	    	    	    	Oligo.setMask(u);
	    	    	    	Oligo.setMask(u2);
	    	    	    	Oligo.setMask(u3);
	    	    	    	Collections.sort(bunk[0]); 
	    	    	    	Iterator it3 = bunk[0].iterator();
	    	    	    	l[1] = 0;
	    	    	    	oligo0 = (Oligo) it3.next ();
	    	    	    	for (; it3.hasNext(); ) {
	    	    	    		oligo[DATA] = (Oligo) it3.next ();
	    	    	    		l[0] = l[1];
	    	    	    		l[1] = oligo[DATA].match(oligo0);
	    	    	    		if (l[0] > l[1]) {
	    	    	    			u1 = l[0];
	    	    	    		} else {
	    	    	    			u1 = l[1];
	    	    	    		}
	    	    	    		if (u1 > oligo0.u[3])
	    	    	    			oligo0.u[3] = u1;
	    	    	    		oligo0 = oligo[DATA];
	    	    	    	}
	    	    	    }
	    	    	}

	    	    }
	    	}
	    }
	    
	    openStreamOut(fl + ".db.s.sq", DATA);
	    saveBunk(DATA);
	    closeStreamOut(DATA);
	    
		System.err.println(" fillU End\n");
    } 

//*/	
	public  void test(String umf) throws Exception {
		Sequencability[] tt;
		tt = new Sequencability[100];
		for (int i = 0; i < 100; i++) {
			tt[i] = new Sequencability(i);
		}
	}
}

